#include once "fbgfx.bi"
#include once "horde3d.bi"

function InitHorde3D(byval w as integer=640,byval h as integer=480) as boolean
  dim as integer bits
  screeninfo ,,bits
  ' create window with OpenGL context
  screenres w,h,bits,,fb.GFX_OPENGL
  if screenptr()=0 then
    screen 0
    print "error: init OpenGL context !"
    beep : sleep : end 1
  end if
  flip
  ' init Horde3D
  var ret = h3dInit()
  if ret=false then h3dutDumpMessages()
  return ret
end function

if InitHorde3D()=false then
  screen 0
  print "error: init Horde3D !"
  beep:sleep:end
end if
' get current screen size
dim as integer w,h
screeninfo w,h

dim as string dumpFile  = command(0) & ".html"
dim as string dumpTitle = "Animation test with: " & *h3dGetVersionString
h3dutSetDumpFile (strptr(dumpFile))
h3dutSetDumpTitle(strptr(dumpTitle))




' Add resources

' Pipeline
'var renderPipeline = h3dAddPipeline("pipelines/deferred.pipeline.xml")
var lightMaterial  = h3dAddMaterial  ("materials/light.material.xml")
var renderPipeline = h3dAddPipeline("pipelines/forward.pipeline.xml")

' Environment
var envRes = h3dAddSceneGraph("models/sphere/sphere.scene.xml")

' Knight
var knightRes      = h3dAddSceneGraph("models/knight/knight.scene.xml")
var knightAnim1Res = h3dAddAnimation("animations/knight_order.anim")
var knightAnim2Res = h3dAddAnimation("animations/knight_attack.anim")

' Particle system
var particleSysRes = h3dAddSceneGraph("particles/particleSys1/particleSys1.scene.xml")


var _fontMatRes  = h3dAddMaterial("overlays/font.material.xml")
var _panelMatRes = h3dAddMaterial("overlays/panel.material.xml")
var _logoMatRes  = h3dAddMaterial("overlays/logo.material.xml")

' Load resources
var contentDir = exepath() + "/Content"
h3dutLoadResourcesFromDisk(strptr(contentDir))

' Add scene nodes

' Add camera
var cam = h3dAddCameraNode(H3DRootNode, "Camera", renderPipeline)
' Setup viewport and render target sizes
h3dSetNodeParamI( cam, H3DCamera_ViewportXI, 0 )
h3dSetNodeParamI( cam, H3DCamera_ViewportYI, 0 )
h3dSetNodeParamI( cam, H3DCamera_ViewportWidthI, w )
h3dSetNodeParamI( cam, H3DCamera_ViewportHeightI, h)

' Set camera parameters (pos, rotation, scale)
h3dSetNodeTransform(cam, 0,5,20, 0,0,0, 1,1,1)

' Set camera parameters (fov,  ratio, near, far)
h3dSetupCameraView( cam, 45.0, w / h, 0.1, 1000.0 )
h3dResizePipelineBuffers( renderPipeline, w, h )


' Add environment
var env = h3dAddNodes(H3DRootNode, envRes)
h3dSetNodeTransform(env, 0, -400, 0, 0, 0, 0, 400, 400, 400)

' Add knight
var knight = h3dAddNodes(H3DRootNode, knightRes)
'                           pos,   rotation,  scale
h3dSetNodeTransform(knight, 0,0,0, 0,180,0,  .1,.1,.1)
h3dSetupModelAnimStage(knight, 0, knightAnim1Res, 0, "", False)
h3dSetupModelAnimStage(knight, 1, knightAnim2Res, 0, "", False)

'' Attach particle system to hand joint
h3dFindNodes(knight, "Bip01_R_Hand", H3DNodeTypes_Joint)
var hand = h3dGetNodeFindResult(0)
var particleSys = h3dAddNodes(hand, particleSysRes)
'                                 pos,  rotation, scale
h3dSetNodeTransform(particleSys, 0,40,0,  0,0,0,  1,1,1)

' Add light source
var light = h3dAddLightNode(H3DRootNode, "Light1", lightMaterial, "LIGHTING", "SHADOWMAP")
'                          pos,    rotation, scale
h3dSetNodeTransform(light, 0,1,5, 0,0,0, 1,1,1)
h3dSetNodeParamF(light, H3DLight_RadiusF, 0, 30)
h3dSetNodeParamF(light, H3DLight_FovF, 0, 90)
h3dSetNodeParamI(light, H3DLight_ShadowMapCountI, 1)
h3dSetNodeParamF(light, H3DLight_ShadowMapBiasF, 0, 0.01)
h3dSetNodeParamF(light, H3DLight_ColorF3, 0, 1) ' red
h3dSetNodeParamF(light, H3DLight_ColorF3, 1, 1) ' green
h3dSetNodeParamF(light, H3DLight_ColorF3, 2, 1) ' blue

const as single FPS = 60
dim as single t

' optinal write out any log messages
h3dutDumpMessages()

' shows the bounding boxes
' h3dSetOption(H3DOptions_DebugViewMode,1)
' usefull if something are wrong with trriangles
' h3dSetOption(H3DOptions_WireframeMode,1)

dim as single coords(...)=>{0.8, 0.8, 0, 1,  _
                            0.8, 1,   0, 0,  _
                            1   , 1,   1, 0, _
                            1   , 0.8, 1, 1}

while inkey()=""

  ' Increase animation time
  t = t + 16 * (1 / FPS)

  ' Do animation blending
  h3dSetModelAnimParams(knight, 0, t, 1) ' order
  h3dSetModelAnimParams(knight, 1, t, 0) ' attack
  h3dUpdateModel(knight,1)
  ' Animate particle systems (several emitters in a group node)
  var cnt = h3dFindNodes(particleSys, "", H3DNodeTypes_Emitter)
  for I as long = 0 to cnt - 1
    h3dUpdateEmitter(h3dGetNodeFindResult(I), 1 / FPS)
  next

  h3dutShowFrameStats(_fontMatRes,_panelMatRes,100)
  h3dShowOverlays(@coords(0), 4, 1,1,1,1, _logoMatRes, 0 )
  h3dutShowText( "Text !", 0, .9, 0.1, 1, 1, 1, _fontMatRes)

  ' Render scene
  h3dRender( cam )

  ' Finish rendering of frame
  h3dFinalizeFrame()
  h3dClearOverlays()
  ' swap buffers
  flip
wend

' free all resources
h3dRelease()
