#include once "fbgfx.bi"
#include once "horde3d.bi"

' a test to create a gemetry resource on the fly

#ifndef NULL
  #define NULL cptr(any ptr,0)
#endif

type vec3f
  as single x,y,z
end type

type vec3s
  as short x,y,z
end type

type vec2f
  as single x,y
end type

function CreateCubeGeometry(byval resName as zstring ptr,byval scale as single=1) as TH3DRes
  dim as long nVertices  = 24
  dim as long nTriangles = 12
  dim as long nTriangleIndices = nTriangles * 3
  dim as vec3f vertexData(...) => { _
  (-0.5*scale, 0.5*scale, 0.5*scale), _ ' front
  (-0.5*scale,-0.5*scale, 0.5*scale), _
  ( 0.5*scale,-0.5*scale, 0.5*scale), _
  ( 0.5*scale, 0.5*scale, 0.5*scale), _
  _
  ( 0.5*scale, 0.5*scale, 0.5*scale), _ ' right
  ( 0.5*scale,-0.5*scale, 0.5*scale), _
  ( 0.5*scale,-0.5*scale,-0.5*scale), _
  ( 0.5*scale, 0.5*scale,-0.5*scale), _ 
  _
  ( 0.5*scale, 0.5*scale,-0.5*scale), _ ' back
  ( 0.5*scale,-0.5*scale,-0.5*scale), _
  (-0.5*scale,-0.5*scale,-0.5*scale), _
  (-0.5*scale, 0.5*scale,-0.5*scale), _
  _
  (-0.5*scale, 0.5*scale,-0.5*scale), _ ' left
  (-0.5*scale,-0.5*scale,-0.5*scale), _
  (-0.5*scale,-0.5*scale, 0.5*scale), _
  (-0.5*scale, 0.5*scale, 0.5*scale), _
  _
  (-0.5*scale, 0.5*scale,-0.5*scale), _ ' top
  (-0.5*scale, 0.5*scale, 0.5*scale), _
  ( 0.5*scale, 0.5*scale, 0.5*scale), _
  ( 0.5*scale, 0.5*scale,-0.5*scale), _
  _
  (-0.5*scale,-0.5*scale, 0.5*scale), _ ' bottom
  (-0.5*scale,-0.5*scale,-0.5*scale), _
  ( 0.5*scale,-0.5*scale,-0.5*scale), _
  ( 0.5*scale,-0.5*scale, 0.5*scale)} 


  dim as vec3s normalData(...) => { _
  (0,0, 32767), (0,0, 32767), (0,0, 32767), (0,0, 32767), _ ' front
  ( 32767,0,0), ( 32767,0,0), ( 32767,0,0), ( 32767,0,0), _ ' right
  (0,0,-32767), (0,0,-32767), (0,0,-32767), (0,0,-32767), _ ' back
  (-32767,0,0), (-32767,0,0), (-32767,0,0), (-32767,0,0), _ ' left
  (0, 32767,0), (0, 32767,0), (0, 32767,0), (0, 32767,0), _ ' top
  (0,-32767,0), (0,-32767,0), (0,-32767,0), (0,-32767,0)}   ' bottom

  dim as vec2f textureData(...) => { _
  (0,1),(0,0),(1,0),(1,1), _ ' front
  (0,1),(0,0),(1,0),(1,1), _ ' right
  (0,1),(0,0),(1,0),(1,1), _ ' back
  (0,1),(0,0),(1,0),(1,1), _ ' left
  (0,1),(0,0),(1,0),(1,1), _ ' top
  (0,1),(0,0),(1,0),(1,1) }  ' bottom

  dim as ulong indexData(...) => { _
   0, 1, 2,  2, 3, 0, _ ' front
   4, 5, 6,  6, 7, 4, _ ' right
   8, 9,10, 10,11, 8, _ ' back
  12,13,14, 14,15,12, _ ' left
  16,17,18, 18,19,16, _ ' top
  20,21,22, 22,23,20}   ' bottom

  return h3dutCreateGeometryRes(resName             , _ ' resource name
                                ubound(vertexData)+1, _ ' num of vertices
                                ubound(indexData) +1, _ ' num of indicies
                                @vertexData(0).x    , _ ' vertex position (single ptr)
                                @indexData(0)       , _ ' indicies (ulong ptr)
                                @normalData(0).x    , _ ' vertex normale   -32767 to +32767 (short ptr) 
                                NULL                , _ ' vertex tangent   -32767 to +32767 (short ptr)
                                NULL                , _ ' vertex bitangent -32767 to +32767 (short ptr)
                                @textureData(0).x   , _ ' vertex texture coords set 0 (single ptr) 
                                NULL)                   ' vertex texture coords set 1 (single ptr) 
end function

function InitHorde3D(byval w as integer=640,byval h as integer=480) as boolean
  dim as integer bits
  screeninfo ,,bits
  ' create window with OpenGL context
  screenres w,h,bits,,fb.GFX_OPENGL
  if screenptr()=0 then
    screen 0
    print "error: init OpenGL context !"
    beep : sleep : end 1
  end if
  flip
  ' init Horde3D
  var ret = h3dInit()
  if ret=false then h3dutDumpMessages()
  return ret
end function

if InitHorde3D()=false then
  screen 0
  print "error: init Horde3D !"
  beep : sleep : end 1
end if
' get current screen size
dim as integer w,h
screeninfo w,h

dim as string dumpFile  = command(0) & ".html"
dim as string dumpTitle = "Create geometry resource with: " & *h3dGetVersionString
h3dutSetDumpFile (strptr(dumpFile))
h3dutSetDumpTitle(strptr(dumpTitle))

' Add resources
var renderPipeline = h3dAddPipeline("pipelines/forward.pipeline.xml")
var platform       = h3dAddSceneGraph("models/platform/platform.scene.xml")
var cubeMaterial   = h3dAddMaterial("models/cube/cube.material.xml")

' Load resources
var contentDir = exepath() + "/Content"
h3dutLoadResourcesFromDisk(strptr(contentDir))

' optional dump log messages
h3dutDumpMessages() ' usefull if loading of any resources are fails

var platformNode = h3dAddNodes(H3DRootNode, platform)

const as integer SQR_CUBES = 10
const as integer MAX_CUBES = SQR_CUBES*SQR_CUBES
dim as TH3DNode cubes(MAX_CUBES-1)
for i as integer =0 to MAX_CUBES-1
  dim as string ID = "cubeGeometry" & i
  var cubeGeometry = CreateCubeGeometry(strptr(id),0.5)
  ID = "cubeModel" & i
  cubes(i) = h3dAddModelNode(H3DRootNode,strptr(id),cubeGeometry)
  ID = "cubeMesh" & i
  h3dAddMeshNode(cubes(i),strptr(id),cubeMaterial,0,36,0,3)
next

var camNode = h3dAddCameraNode(H3DRootNode, "Camera", RenderPipeline)
h3dSetNodeTransform(camNode, 0,3,8+SQR_CUBES/2, -30*0,0,0, 1,1,1)

h3dSetNodeTransform(platformNode, 0,0,0,0,0,0, 1/100*10,1,1/100*10)

' Setup viewport and render target sizes
h3dSetNodeParamI(camNode, H3DCamera_ViewportWidthI , w)
h3dSetNodeParamI(camNode, H3DCamera_ViewportHeightI, h)
h3dResizePipelineBuffers( RenderPipeline, w,h)

' Set camera parameters (fov,  ratio, near, far)
h3dSetupCameraView( camNode, 45.0, w/h, 0.1, 1000.0 )

' Add light
var lightNode = h3dAddLightNode(H3DRootNode, "Light1", 0, "LIGHTING", "SHADOWMAP")
h3dSetNodeTransform(lightNode, 0,15,0, -90,0,0, 1,1,1)
h3dSetNodeParamI(lightNode, H3DLight_ShadowMapCountI, 1)

' set cube position
for i as integer =0 to MAX_CUBES-1
  var z =  (i  \  SQR_CUBES) - SQR_CUBES/2
  var x =  (i mod SQR_CUBES) - SQR_CUBES/2 + .5
  h3dSetNodeTransform(cubes(i),x,.25,z, 0,0,0, 1,1,1)
next

dim as integer frames,xRot,yRot,zRot
while inkey()=""
  ' rotate the cube
  for i as integer =0 to MAX_CUBES-1
    var z = (i  \  SQR_CUBES) - SQR_CUBES/2
    var x = (i mod SQR_CUBES) - SQR_CUBES/2 + .5
    var d = sqr(x*x+z*z)
    if d < SQR_CUBES/4 then
      h3dSetNodeTransform(cubes(i),x,.25+SQR_CUBES/4*.3 - d*.3,z, xRot,yRot,zRot, 1,1,1)
    end if
  next
  ' Render scene
  h3dRender(camNode)
  ' Finish rendering of frame
  h3dFinalizeFrame()
  ' swap buffers
  flip
  h3dutDumpMessages()
  frames+=1
  xRot=(xRot+1) mod 360
  if frames mod 2=0 then yRot=(yRot+1) mod 360
  if frames mod 3=0 then zRot=(zRot+1) mod 360
wend

' free all resources
h3dRelease()
