#ifndef __horde3d_bi__
#define __horde3d_bi__

' Horde3D FreeBASIC include file version 1.0 Beta 5

#ifdef __FB_WIN32__
#libpath "lib/win"
#endif

#ifndef __FB_64BIT__
  #inclib "horde3D-32"
#else
  #inclib "horde3d-64"
#endif


' Group: Typedefs and constants
type TH3DRes  as long
type TH3DNode as long

const as TH3DNode H3DRootNode = 1

' Group: Enumerations

' Enum: H3DOptions used by: h3dGetOption() h3dSetOption()
' The available engine option parameters.
'  MaxLogLevel         - Defines the maximum log level; only messages which are smaller or equal to this value
'                        (hence more important) are published in the message queue. (Default: 4)
'  MaxNumMessages      - Defines the maximum number of messages that can be stored in the message queue (Default: 512)
'  TrilinearFiltering  - Enables or disables trilinear filtering for textures. (Values: 0, 1; Default: 1)
'  MaxAnisotropy       - Sets the maximum quality for anisotropic filtering. (Values: 1, 2, 4, 8; Default: 1)
'  TexCompression      - Enables or disables texture compression; only affects textures that are
'                        loaded after setting the option. (Values: 0, 1; Default: 0)
'  SRGBLinearization   - Eanbles or disables gamma-to-linear-space conversion of input textures that are tagged as sRGB (Values: 0, 1; Default: 0)
'  LoadTextures        - Enables or disables loading of textures referenced by materials; this can be useful to reduce
'                        loading times for testing. (Values: 0, 1; Default: 1)
'  FastAnimation       - Disables or enables inter-frame interpolation for animations. (Values: 0, 1; Default: 1)
'  ShadowMapSize       - Sets the size of the shadow map buffer (Values: 128, 256, 512, 1024, 2048; Default: 1024)
'  SampleCount         - Maximum number of samples used for multisampled render targets; only affects pipelines
'                        that are loaded after setting the option. (Values: 0, 2, 4, 8, 16; Default: 0)
'  WireframeMode       - Enables or disables wireframe rendering
'  DebugViewMode       - Enables or disables debug view where geometry is rendered in wireframe without shaders and
'                        lights are visualized using their screen space bounding box. (Values: 0, 1; Default: 0)
'  DumpFailedShaders   - Enables or disables storing of shader code that failed to compile in a text file; this can be
'                        useful in combination with the line numbers given back by the shader compiler. (Values: 0, 1; Default: 0)
'  GatherTimeStats     - Enables or disables gathering of time stats that are useful for profiling (Values: 0, 1; Default: 1)
'/
type TH3DOptions as ulong
const as TH3DOptions H3DOptions_MaxLogLevel        =  1
const as TH3DOptions H3DOptions_MaxNumMessages     =  2
const as TH3DOptions H3DOptions_TrilinearFiltering =  3
const as TH3DOptions H3DOptions_MaxAnisotropy      =  4
const as TH3DOptions H3DOptions_TexCompression     =  5
const as TH3DOptions H3DOptions_SRGBLinearization  =  6
const as TH3DOptions H3DOptions_LoadTextures       =  7
const as TH3DOptions H3DOptions_FastAnimation      =  8
const as TH3DOptions H3DOptions_ShadowMapSize      =  9
const as TH3DOptions H3DOptions_SampleCount        = 10
const as TH3DOptions H3DOptions_WireframeMode      = 11
const as TH3DOptions H3DOptions_DebugViewMode      = 12
const as TH3DOptions H3DOptions_DumpFailedShaders  = 13
const as TH3DOptions H3DOptions_GatherTimeStats    = 14


' Enum: H3DStats used by: h3dGetStat()
' The available engine statistic parameters.

' TriCount          - Number of triangles that were pushed to the renderer
' BatchCount        - Number of batches (draw calls)
' LightPassCount    - Number of lighting passes
' FrameTime         - Time in ms between two h3dFinalizeFrame calls

' AnimationTime     - CPU time in ms spent for animation
' GeoUpdateTime     - CPU time in ms spent for software skinning and morphing
' ParticleSimTime   - CPU time in ms spent for particle simulation and updates

' FwdLightsGPUTime  - GPU time in ms spent for forward lighting passes
' DefLightsGPUTime  - GPU time in ms spent for drawing deferred light volumes
' ShadowsGPUTime    - GPU time in ms spent for generating shadow maps
' ParticleGPUTime   - GPU time in ms spent for drawing particles

' TextureVMem       - Estimated amount of video memory used by textures (in Mb)
' GeometryVMem      - Estimated amount of video memory used by geometry (in Mb)
type TH3DStats as ulong
const as TH3DStats H3DStats_TriCount         = 100
const as TH3DStats H3DStats_BatchCount       = 101
const as TH3DStats H3DStats_LightPassCount   = 102
const as TH3DStats H3DStats_FrameTime        = 103
const as TH3DStats H3DStats_AnimationTime    = 104
const as TH3DStats H3DStats_GeoUpdateTime    = 105
const as TH3DStats H3DStats_ParticleSimTime  = 106
const as TH3DStats H3DStats_FwdLightsGPUTime = 107
const as TH3DStats H3DStats_DefLightsGPUTime = 108
const as TH3DStats H3DStats_ShadowsGPUTime   = 109
const as TH3DStats H3DStats_ParticleGPUTime  = 110
const as TH3DStats H3DStats_TextureVMem      = 111
const as TH3DStats H3DStats_GeometryVMem     = 112

' Enum: H3DResTypes used by: ' h3dAddResource() h3dGetResType() h3dFindResource()  
' The available resource types.
' Undefined       - An undefined resource, returned by getResourceType in case of error
' SceneGraph      - Scene graph subtree stored in XML format
' Geometry        - Geometrical data containing bones, vertices and triangles
' Animation       - Animation data
' Material        - Material script
' Code            - Text block containing shader source code
' Shader          - Shader program
' Texture         - Texture map
' ParticleEffect  - Particle configuration
' Pipeline        - Rendering pipeline
type TH3DResTypes as ulong
const as TH3DResTypes H3DResTypes_Undefined      = 0
const as TH3DResTypes H3DResTypes_SceneGraph     = 1
const as TH3DResTypes H3DResTypes_Geometry       = 2
const as TH3DResTypes H3DResTypes_Animation      = 3
const as TH3DResTypes H3DResTypes_Material       = 4
const as TH3DResTypes H3DResTypes_Code           = 5
const as TH3DResTypes H3DResTypes_Shader         = 6
const as TH3DResTypes H3DResTypes_Texture        = 7
const as TH3DResTypes H3DResTypes_ParticleEffect = 8
const as TH3DResTypes H3DResTypes_Pipeline       = 9

' Enum: H3DResFlags used by: h3dAddResource()
' The available flags used when adding a resource.
' NoQuery           - Excludes resource from being listed by queryUnloadedResource function.
' NoTexCompression  - Disables texture compression for Texture resource.
' NoTexMipmaps      - Disables generation of mipmaps for Texture resource.
' TexCubemap        - Sets Texture resource to be a cubemap.
' TexDynamic        - Enables more efficient updates of Texture resource streams.
' TexRenderable     - Makes Texture resource usable as render target.
' TexSRGB           - Indicates that Texture resource is in sRGB color space and should be converted to linear space when being sampled.
type TH3DResFlags as ulong
const as TH3DResFlags H3DResFlags_Default          = 0
const as TH3DResFlags H3DResFlags_NoQuery          = 1
const as TH3DResFlags H3DResFlags_NoTexCompression = 2
const as TH3DResFlags H3DResFlags_NoTexMipmaps     = 4
const as TH3DResFlags H3DResFlags_TexCubemap       = 8
const as TH3DResFlags H3DResFlags_TexDynamic       = 16
const as TH3DResFlags H3DResFlags_TexRenderable    = 32
const as TH3DResFlags H3DResFlags_TexSRGB          = 64
  
' Enum: H3DFormats used by: h3dCreateTexture() 
' The available resource stream formats.
' Unknown      - Unknown format
' TEX_BGRA8    - 8-bit BGRA texture
' TEX_DXT1     - DXT1 compressed texture
' TEX_DXT3     - DXT3 compressed texture
' TEX_DXT5     - DXT5 compressed texture
' TEX_RGBA16F  - Half float RGBA texture
' TEX_RGBA32F  - Float RGBA texture
type TH3DFormats as ulong
const as TH3DFormats H3DFormats_Unknown     = 0
const as TH3DFormats H3DFormats_TEX_BGRA8   = 1
const as TH3DFormats H3DFormats_TEX_DXT1    = 2
const as TH3DFormats H3DFormats_TEX_DXT3    = 3
const as TH3DFormats H3DFormats_TEX_DXT5    = 4
const as TH3DFormats H3DFormats_TEX_RGBA16F = 5
const as TH3DFormats H3DFormats_TEX_RGBA32F = 6

' Enum: H3DGeoRes
' The available Geometry resource accessors.

' GeometryElem         - Base element
' GeoIndexCountI       - Number of indices [read-only]
' GeoVertexCountI      - Number of vertices [read-only]
' GeoIndices16I        - Flag indicating whether index data is 16 or 32 bit [read-only]
' GeoIndexStream       - Triangle index data (ushort or ulong, depending on flag)
' GeoVertPosStream     - Vertex position data (single x, y, z)
' GeoVertTanStream     - Vertex tangent frame data (single nx, ny, nz, tx, ty, tz, tw)
' GeoVertStaticStream  - Vertex static attribute data (single u0, v0, single(3) jointIndices, single(3) jointWeights, single u1, v1)
type TH3DGeoRes as ulong
const as TH3DGeoRes H3DGeoRes_GeometryElem        = 200 ' (h3dMapResStream)
const as TH3DGeoRes H3DGeoRes_GeoIndexCountI      = 201
const as TH3DGeoRes H3DGeoRes_GeoVertexCountI     = 202
const as TH3DGeoRes H3DGeoRes_GeoIndices16I       = 203

const as TH3DGeoRes H3DGeoRes_GeoIndexStream      = 204 ' (h3dMapResStream)
const as TH3DGeoRes H3DGeoRes_GeoVertPosStream    = 205 ' (h3dMapResStream)
const as TH3DGeoRes H3DGeoRes_GeoVertTanStream    = 206 ' (h3dMapResStream)
const as TH3DGeoRes H3DGeoRes_GeoVertStaticStream = 207 ' (h3dMapResStream)

' Enum: H3DAnimRes
' The available Animation resource accessors.
' EntityElem      - Stored animation entities (joints and meshes)
' EntFrameCountI  - Number of frames stored for a specific entity [read-only]
type TH3DAnimRes as ulong
const as TH3DAnimRes H3DAnimRes_EntityElem     = 300
const as TH3DAnimRes H3DAnimRes_EntFrameCountI = 301

' Enum: H3DMatRes
' The available Material resource accessors.
' MaterialElem  - Base element
' SamplerElem   - Sampler element
' UniformElem   - Uniform element
' MatClassStr   - Material class
' MatLinkI      - Material resource that is linked to this material
' MatShaderI    - Shader resource
' SampNameStr   - Name of sampler [read-only]
' SampTexResI   - Texture resource bound to sampler
' UnifNameStr   - Name of uniform [read-only]
' UnifValueF4   - Value of uniform (a, b, c, d)
type TH3DMatRes as ulong
const as TH3DMatRes H3DMatRes_MaterialElem = 400
const as TH3DMatRes H3DMatRes_SamplerElem  = 401
const as TH3DMatRes H3DMatRes_UniformElem  = 402
const as TH3DMatRes H3DMatRes_MatClassStr  = 403
const as TH3DMatRes H3DMatRes_MatLinkI     = 404
const as TH3DMatRes H3DMatRes_MatShaderI   = 405
const as TH3DMatRes H3DMatRes_SampNameStr  = 406
const as TH3DMatRes H3DMatRes_SampTexResI  = 407
const as TH3DMatRes H3DMatRes_UnifNameStr  = 408
const as TH3DMatRes H3DMatRes_UnifValueF4  = 409


' Enum: H3DShaderRes
' The available Shader resource accessors.
' ContextElem     - Context element
' SamplerElem     - Sampler element
' UniformElem     - Uniform element
' ContNameStr     - Name of context [read-only]
' SampNameStr     - Name of sampler [read-only]
' UnifNameStr     - Name of uniform [read-only]
' UnifSizeI       - Size (number of components) of uniform [read-only]
' UnifDefValueF4  - Default value of uniform (a, b, c, d)
type TH3DShaderRes as ulong
const as TH3DShaderRes H3DShaderRes_ContextElem    = 601
const as TH3DShaderRes H3DShaderRes_SamplerElem    = 602
const as TH3DShaderRes H3DShaderRes_UniformElem    = 603
const as TH3DShaderRes H3DShaderRes_ContNameStr    = 604
const as TH3DShaderRes H3DShaderRes_SampNameStr    = 605
const as TH3DShaderRes H3DShaderRes_UnifNameStr    = 606
const as TH3DShaderRes H3DShaderRes_UnifSizeI      = 607
const as TH3DShaderRes H3DShaderRes_UnifDefValueF4 = 608

' Enum: H3DTexRes
' The available Texture resource accessors.
' TextureElem     - Base element
' ImageElem       - Subresources of the texture. A texture consists, depending on the type,
'                   of a number of equally sized slices which again can have a fixed number
'                   of mipmaps. Each image element represents the base image of a slice or
'                   a single mipmap level of the corresponding slice.
' TexFormatI      - Texture format [read-only]
' TexSliceCountI  - Number of slices (1 for 2D texture and 6 for cubemap) [read-only]
' ImgWidthI       - Image width [read-only]
' ImgHeightI      - Image height [read-only]
' ImgPixelStream  - Pixel data of an image. The data layout matches the layout specified
'                   by the texture format with the exception that half-float is converted
'                   to float. The first element in the data array corresponds to the lower left corner.
type TH3DTexRes as ulong
const as TH3DTexRes H3DTexRes_TextureElem    = 700
const as TH3DTexRes H3DTexRes_ImageElem      = 701 ' (h3dMapResStream)
const as TH3DTexRes H3DTexRes_TexFormatI     = 702
const as TH3DTexRes H3DTexRes_TexSliceCountI = 703
const as TH3DTexRes H3DTexRes_ImgWidthI      = 704
const as TH3DTexRes H3DTexRes_ImgHeightI     = 705
const as TH3DTexRes H3DTexRes_ImgPixelStream = 706  ' (h3dMapResStream)

' Enum: H3DPartEffRes
' The available ParticleEffect resource accessors.
' ParticleElem     - General particle configuration
' ChanMoveVelElem  - Velocity channel
' ChanRotVelElem   - Angular velocity channel
' ChanSizeElem     - Size channel
' ChanColRElem     - Red color component channel
' ChanColGElem     - Green color component channel
' ChanColBElem     - Blue color component channel
' ChanColAElem     - Alpha channel
' PartLifeMinF     - Minimum value of random life time (in seconds)
' PartLifeMaxF     - Maximum value of random life time (in seconds)
' ChanStartMinF    - Minimum for selecting initial random value of channel
' ChanStartMaxF    - Maximum for selecting initial random value of channel
'  ChanEndRateF     - Remaining percentage of initial value when particle is dying
type TH3DPartEffRes as ulong
const as TH3DPartEffRes H3DPartEffRes_ParticleElem    = 800
const as TH3DPartEffRes H3DPartEffRes_ChanMoveVelElem = 801
const as TH3DPartEffRes H3DPartEffRes_ChanRotVelElem  = 802
const as TH3DPartEffRes H3DPartEffRes_ChanSizeElem    = 803
const as TH3DPartEffRes H3DPartEffRes_ChanColRElem    = 804
const as TH3DPartEffRes H3DPartEffRes_ChanColGElem    = 805
const as TH3DPartEffRes H3DPartEffRes_ChanColBElem    = 806
const as TH3DPartEffRes H3DPartEffRes_ChanColAElem    = 807
const as TH3DPartEffRes H3DPartEffRes_PartLifeMinF    = 808
const as TH3DPartEffRes H3DPartEffRes_PartLifeMaxF    = 809
const as TH3DPartEffRes H3DPartEffRes_ChanStartMinF   = 810
const as TH3DPartEffRes H3DPartEffRes_ChanStartMaxF   = 811
const as TH3DPartEffRes H3DPartEffRes_ChanEndRateF    = 812
const as TH3DPartEffRes H3DPartEffRes_ChanDragElem    = 813

' Enum: H3DPipeRes
' The available Pipeline resource accessors.
' StageElem         - Pipeline stage
' StageNameStr      - Name of stage [read-only]
' StageActivationI  - Flag indicating whether stage is active
type TH3DPipeRes as ulong
const as TH3DPipeRes H3DPipeRes_StageElem        = 900
const as TH3DPipeRes H3DPipeRes_StageNameStr     = 901
const as TH3DPipeRes H3DPipeRes_StageActivationI = 902

' Enum: H3DNodeTypes
' The available scene node types.
' Undefined  - An undefined node type, returned by getNodeType in case of error
' Group      - Group of different scene nodes
' Model      - 3D model with optional skeleton
' Mesh       - Subgroup of a model with triangles of one material
' Joint      - Joint for skeletal animation
' Light      - Light source
' Camera     - Camera giving view on scene
' Emitter    - Particle system emitter
type TH3DNodeTypes as ulong
const as TH3DNodeTypes H3DNodeTypes_Undefined = 0
const as TH3DNodeTypes H3DNodeTypes_Group     = 1
const as TH3DNodeTypes H3DNodeTypes_Model     = 2
const as TH3DNodeTypes H3DNodeTypes_Mesh      = 3
const as TH3DNodeTypes H3DNodeTypes_Joint     = 4
const as TH3DNodeTypes H3DNodeTypes_Light     = 5
const as TH3DNodeTypes H3DNodeTypes_Camera    = 6
const as TH3DNodeTypes H3DNodeTypes_Emitter   = 7

' Enum: H3DNodeFlags
' The available scene node flags.
' NoDraw         - Excludes scene node from all rendering
' NoCastShadow   - Excludes scene node from list of shadow casters
' NoRayQuery     - Excludes scene node from ray intersection queries
' Inactive       - Deactivates scene node so that it is completely ignored
type TH3DNodeFlags as ulong
const as TH3DNodeFlags H3DNodeFlags_NoDraw       = 1
const as TH3DNodeFlags H3DNodeFlags_NoCastShadow = 2
const as TH3DNodeFlags H3DNodeFlags_NoRayQuery   = 4
const as TH3DNodeFlags H3DNodeFlags_Inactive     = 7  ' NoDraw or NoCastShadow or NoRayQuery

' Enum: H3DNodeParams
' The available scene node parameters.
' NameStr        - Name of the scene node
' AttachmentStr  - Optional application-specific meta data for a node encapsulated in an 'Attachment' XML string
type TH3DNodeParams as ulong
const as TH3DNodeParams H3DNodeParams_NameStr       = 1
const as TH3DNodeParams H3DNodeParams_AttachmentStr = 2

' Enum: H3DModel
' The available Model node parameters
' GeoResI      - Geometry resource used for the model
' SWSkinningI  - Enables or disables software skinning (default: 0)
' LodDist1F    - Distance to camera from which on LOD1 is used >  0.0  (default: infinite)
' LodDist2F    - Distance to camera from which on LOD2 is used >= LOD1 (default: infinite)
' LodDist3F    - Distance to camera from which on LOD3 is used >= LOD2 (default: infinite)
' LodDist4F    - Distance to camera from which on LOD4 is used >= LOD3 (default: infinite)
type TH3DModel as long
const as TH3DModel H3DModel_GeoResI     = 200
const as TH3DModel H3DModel_SWSkinningI = 201
const as TH3DModel H3DModel_LodDist1F   = 202
const as TH3DModel H3DModel_LodDist2F   = 203
const as TH3DModel H3DModel_LodDist3F   = 204
const as TH3DModel H3DModel_LodDist4F   = 205

' Enum: H3DMesh
' The available Mesh node parameters.
' MatResI      - Material resource used for the mesh
' BatchStartI  - First triangle index of mesh in Geometry resource of parent Model node [read-only]
' BatchCountI  - Number of triangle indices used for drawing mesh [read-only]
' VertRStartI  - First vertex in Geometry resource of parent Model node [read-only]
' VertREndI    - Last vertex in Geometry resource of parent Model node [read-only]
' LodLevelI    - LOD level of Mesh; the mesh is only rendered if its LOD level corresponds to
'                the model's current LOD level which is calculated based on the LOD distances (default: 0)
type TH3DMesh as ulong
const as TH3DMesh H3DMesh_MatResI     = 300
const as TH3DMesh H3DMesh_BatchStartI = 301
const as TH3DMesh H3DMesh_BatchCountI = 302
const as TH3DMesh H3DMesh_VertRStartI = 303
const as TH3DMesh H3DMesh_VertREndI   = 304
const as TH3DMesh H3DMesh_LodLevelI   = 305

' Enum: H3DJoint
' The available Joint node parameters.
' JointIndexI  - Index of joint in Geometry resource of parent Model node [read-only]
type TH3DJoint as ulong
const as TH3DJoint H3DJoint_JointIndexI = 400

' Enum: H3DLight
' The available Light node parameters.
' MatResI             - Material resource used for the light
' RadiusF             - Radius of influence (default: 100.0)
' FovF                - Field of view (FOV) angle (default: 90.0)
' ColorF3             - Diffuse color RGB (default: 1.0, 1.0, 1.0)
' ColorMultiplierF    - Diffuse color multiplier for altering intensity, mainly useful for HDR (default: 1.0)
' ShadowMapCountI     - Number of shadow maps used for light source (values: 0, 1, 2, 3, 4; default: 0)]
' ShadowSplitLambdaF  - Constant determining segmentation of view frustum for Parallel Split Shadow Maps (default: 0.5)
' ShadowMapBiasF      - Bias value for shadow mapping to reduce shadow acne (default: 0.005)
' LightingContextStr  - Name of shader context used for computing lighting
' ShadowContextStr    - Name of shader context used for generating shadow map
type TH3DLight as ulong
const as TH3DLight H3DLight_MatResI            = 500
const as TH3DLight H3DLight_RadiusF            = 501
const as TH3DLight H3DLight_FovF               = 502
const as TH3DLight H3DLight_ColorF3            = 503
const as TH3DLight H3DLight_ColorMultiplierF   = 504
const as TH3DLight H3DLight_ShadowMapCountI    = 505
const as TH3DLight H3DLight_ShadowSplitLambdaF = 506
const as TH3DLight H3DLight_ShadowMapBiasF     = 507
const as TH3DLight H3DLight_LightingContextStr = 508
const as TH3DLight H3DLight_ShadowContextStr   = 509

' Enum: H3DCamera
' The available Camera node parameters.
' PipeResI         - Pipeline resource used for rendering
' OutTexResI       - 2D Texture resource used as output buffer (can be 0 to use main framebuffer) (default: 0)
' OutBufIndexI     - Index of the output buffer for stereo rendering (values: 0 for left eye, 1 for right eye) (default: 0)
' LeftPlaneF       - Coordinate of left plane relative to near plane center (default: -0.055228457)
' RightPlaneF      - Coordinate of right plane relative to near plane center (default: 0.055228457)
' BottomPlaneF     - Coordinate of bottom plane relative to near plane center (default: -0.041421354f)
' TopPlaneF        - Coordinate of top plane relative to near plane center (default: 0.041421354f)
' NearPlaneF       - Distance of near clipping plane (default: 0.1)
' FarPlaneF        - Distance of far clipping plane (default: 1000)
' ViewportXI       - Position x-coordinate of the lower left corner of the viewport rectangle (default: 0)
' ViewportYI       - Position y-coordinate of the lower left corner of the viewport rectangle (default: 0)
' ViewportWidthI   - Width of the viewport rectangle (default: 320)
' ViewportHeightI  - Height of the viewport rectangle (default: 240)
' OrthoI           - Flag for setting up an orthographic frustum instead of a perspective one (default: 0)
' OccCullingI      - Flag for enabling occlusion culling (default: 0)
type TH3DCamera as ulong
const as TH3DCamera H3DCamera_PipeResI        = 600
const as TH3DCamera H3DCamera_OutTexResI      = 601
const as TH3DCamera H3DCamera_OutBufIndexI    = 602
const as TH3DCamera H3DCamera_LeftPlaneF      = 603
const as TH3DCamera H3DCamera_RightPlaneF     = 604
const as TH3DCamera H3DCamera_BottomPlaneF    = 605
const as TH3DCamera H3DCamera_TopPlaneF       = 606
const as TH3DCamera H3DCamera_NearPlaneF      = 607
const as TH3DCamera H3DCamera_FarPlaneF       = 608
const as TH3DCamera H3DCamera_ViewportXI      = 609
const as TH3DCamera H3DCamera_ViewportYI      = 610
const as TH3DCamera H3DCamera_ViewportWidthI  = 611
const as TH3DCamera H3DCamera_ViewportHeightI = 612
const as TH3DCamera H3DCamera_OrthoI          = 613
const as TH3DCamera H3DCamera_OccCullingI     = 614

' Enum: H3DEmitter
' The available Emitter node parameters.
' MatResI        - Material resource used for rendering
' PartEffResI    - ParticleEffect resource which configures particle properties
' MaxCountI      - Maximal number of particles living at the same time
' RespawnCountI  - Number of times a single particle is recreated after dying (-1 for infinite)
' DelayF         - Time in seconds before emitter begins creating particles (default: 0.0)
' EmissionRateF  - Maximal number of particles to be created per second (default: 0.0)
' SpreadAngleF   - Angle of cone for random emission direction (default: 0.0)
' ForceF3        - Force vector XYZ applied to particles (default: 0.0, 0.0, 0.0)
type TH3DEmitter as ulong
const as TH3DEmitter H3DEmitter_MatResI       = 700
const as TH3DEmitter H3DEmitter_PartEffResI   = 701
const as TH3DEmitter H3DEmitter_MaxCountI     = 702
const as TH3DEmitter H3DEmitter_RespawnCountI = 703
const as TH3DEmitter H3DEmitter_DelayF        = 704
const as TH3DEmitter H3DEmitter_EmissionRateF = 705
const as TH3DEmitter H3DEmitter_SpreadAngleF  = 706
const as TH3DEmitter H3DEmitter_ForceF3       = 707

' Enum: H3DEXTTerrain
' The available Terrain node parameters.
' HeightTexResI  - Height map texture must be square and a power of two [write-only]
' MatResI        - Material resource used for rendering the terrain
' MeshQualityF   - Constant controlling the overall resolution of the terrain mesh (default: 50.0)
' SkirtHeightF   - Height of the skirts used to hide cracks (default: 0.1)
' BlockSizeI     - Size of a terrain block that is drawn in a single render call; must be 2^n+1 (default: 17)
type H3DEXTTerrain as ulong
const as H3DEXTTerrain H3DEXTTerrain_HeightTexResI = 10000
const as H3DEXTTerrain H3DEXTTerrain_MatResI       = 10001
const as H3DEXTTerrain H3DEXTTerrain_MeshQualityF  = 10002
const as H3DEXTTerrain H3DEXTTerrain_SkirtHeightF  = 10003
const as H3DEXTTerrain H3DEXTTerrain_BlockSizeI    = 10004


' Enum: H3DModelUpdateFlags
' The available flags for h3dUpdateModel.
' Animation  - Apply animation
' Geometry   - Apply morphers and software skinning
type H3DModelUpdateFlags as ulong
const as H3DModelUpdateFlags  H3DModelUpdateFlags_Animation = 1
const as H3DModelUpdateFlags  H3DModelUpdateFlags_Geometry  = 2

extern "C" 

' --- Basic functions ---

' This function returns a pointer to a string containing the current version of Horde3D.
declare function h3dGetVersionString() as const zstring ptr

' This function initializes the graphics engine and makes it ready for use.  
' It has to be the first call to the engine except for getVersionString. 
' In order to successfully initialize the engine the calling application must provide a valid OpenGL context.
' The function can be called several times on different rendering contexts in order to initialize them.
declare function h3dInit() as Boolean

' This function releases the engine and frees all objects and associated memory.  
' It should be called when the application is destroyed.
declare sub      h3dRelease()

' This function checks if a specified extension is contained in the DLL/shared object of the engine.
declare function h3dCheckExtension(extensionName as const zstring ptr) as Boolean

' This function checks if an error occured in a previous API function call.
declare function h3dGetError() as Boolean

' This is the main function of the engine.
' It executes all the rendering, animation and other tasks.
' The function can be called several times per frame, for example in order to write to different output buffers.
declare sub      h3dRender(byval cameraNode as TH3DNode) 

' This function tells the engine that the current frame is finished and that all subsequent rendering operations will be for the next frame.
declare sub      h3dFinalizeFrame()

' This function removes all nodes from the scene graph except the root node and releases all resources.
' !!! Warning: !!! All resource and node IDs are invalid after calling this function
declare sub      h3dClear() 

' General stuff
' This function returns the next message string from the message queue 
' and writes additional information to the specified variables.
' If no message is left over in the queue an empty string is returned.
' !!! level pointer, time pointer can be NULL !!
declare function h3dGetMessage(byval level as long ptr, byval pTime as single ptr=0) as const zstring ptr 

' This function get/set a specified option parameter.
declare function h3dGetOption(byval param as TH3DOptions) as single
declare function h3dSetOption(byval param as TH3DOptions, byval fValue as single) as Boolean

' This function returns the value of the specified statistic.
' The reset flag makes it possible to reset the statistic value after reading.
declare function h3dGetStat(byval param as TH3DStats,byval blnReset as Boolean=true) as single

' This function displays one or more overlays with a specified material and color.
' An overlay is a screen-space quad that can be used to render 2D GUI elements.
' The overlay coordinate system has its origin (0, 0) at the top-left corner of the screen 
' and its maximum (aspect, 1) at the bottom-right corner.
' As the x coordinate of the maximum corresponds to the aspect ratio of the viewport, 
' the size of overlays can always be the same, even when different screen formats (standard 4:3, widescreen 16:9, etc.) are used.
' Texture coordinates are using a system where the coordinates (0, 0) correspond to the lower left corner of the image.  
' Overlays are drawn in the order in which they are pushed using this function.
' Overlays with the same state will be batched together, so it can make sense to group 
' overlays that have the same material, color  and flags in order to achieve best performance.  
' Note that the overlays have to be removed manually using the function h3dClearOverlays.
' Parameters:
' pVerts                  vertex data (x, y, u, v), interpreted as quads
' vertCount               number of vertices (must be multiple of 4)
' colR, colG, colB, colA  color (and transparency) of overlays
' materialRes             material resource used for rendering
' flags                   overlay flags (reserved for future use)
declare sub      h3dShowOverlays(byval verts as single ptr, byval vertCount as long, _
                                 byval r as single, byval g as single, byval b as single, byval a as single, _
                                 byval materialRes as TH3DRes, byval flags as ulong=0) 
' This function removes all overlays that were added with h3dShowOverlays.
declare sub      h3dClearOverlays()

' --- General resource management functions ---
' This function returns the type of a specified resource.
' If the resource handle is invalid, the function returns the resource type Unknown.
declare function h3dGetResType(byval res as TH3DRes) as TH3DResTypes

' This function returns a pointer to the name of a specified resource.
' If the resource handle is invalid, the function returns an empty string.
' !!! Important Note: !!! 
' The pointer is const and allows only read access to the data.
' Do never try to modify the data of the pointer since that can corrupt the engines internal states!
declare function h3dGetResName(byval res as TH3DRes) as const zstring ptr 

' This function searches the next resource of the specified type and returns its handle.
' The search begins after the specified start handle.
' If a further resource of the queried type does not exist, a zero handle is returned.
' The function can be used to iterate over all resources of a given type by using as start the return value of the previous iteration step.  
' The first iteration step should start at 0 and iteration can be ended when the function returns 0.
declare function h3dGetNextResource(byval ResType as TH3DResTypes, byval start as TH3DRes) as TH3DRes

' This function searches the resource of the specified type and name and returns its handle.
' If the resource is not available in the resource manager a zero handle is returned.
declare function h3dFindResource(byval ResType as TH3DResTypes, byval resname as const zstring ptr) as TH3DRes
#define h3dFindSceneGraph(resname)     h3dFindResource(H3DResTypes_SceneGraph,resname)
#define h3dFindGeometry(resname)       h3dFindResource(H3DResTypes_Geometry,resname)
#define h3dFindAnimation(resname)      h3dFindResource(H3DResTypes_Animation,resname)
#define h3dFindMaterial(resname)       h3dFindResource(H3DResTypes_Material,resname)
#define h3dFindCode(resname)           h3dFindResource(H3DResTypes_Code,resname)
#define h3dFindShader(resname)         h3dFindResource(H3DResTypes_Shader,resname)
#define h3dFindTexture(resname)        h3dFindResource(H3DResTypes_Texture,resname)
#define h3dFindParticleEffect(resname) h3dFindResource(H3DResTypes_ParticleEffect,resname)
#define h3dFindPipeline(resname)       h3dFindResource(H3DResTypes_Pipeline,resname)



' This function tries to add a resource of a specified type and name to the resource manager.
' If a resource of the same type and name is already existing, the handle to the existing resource is returned 
' instead of creating a new one and the user reference count of the resource is increased.
declare function h3dAddResource(byval ResType as TH3DResTypes, byval resname as const zstring ptr, byval flags as TH3DResFlags=H3DResFlags_Default) as TH3DRes

#define h3dAddSceneGraph(resname)     h3dAddResource(H3DResTypes_SceneGraph,resname)
#define h3dAddGeometry(resname)       h3dAddResource(H3DResTypes_Geometry,resname)
#define h3dAddAnimation(resname)      h3dAddResource(H3DResTypes_Animation,resname)
#define h3dAddMaterial(resname)       h3dAddResource(H3DResTypes_Material,resname)
#define h3dAddCode(resname)           h3dAddResource(H3DResTypes_Code,resname)
#define h3dAddShader(resname)         h3dAddResource(H3DResTypes_Shader,resname)
#define h3dAddTexture(resname)        h3dAddResource(H3DResTypes_Texture,resname)
#define h3dAddParticleEffect(resname) h3dAddResource(H3DResTypes_ParticleEffect,resname)
#define h3dAddPipeline(resname)       h3dAddResource(H3DResTypes_Pipeline,resname)


' This function duplicates a specified resource.  
' In the cloning process a new resource with the specified name is added to the resource manager 
' and filled with the data of the specified source resource.  
' If the specified name for the new resource is already in use, the function fails and returns 0.
' If the name string is empty, a unique name for the resource is generated automatically.
declare function h3dCloneResource(byval sourceRes as TH3DRes, byval newResname as const zstring ptr) as TH3DRes

' This function decreases the user reference count of a specified resource.
' When the user reference count is zero and there are no internal references, 
' the resource can be released and removed using the API fuction releaseUnusedResources.
declare function h3dRemoveResource(byval res as TH3DRes) as long

' This function checks if the specified resource has been successfully loaded.
declare function h3dIsResLoaded(byval res as TH3DRes) as Boolean

' This function loads data for a resource that was previously added to the resource manager.
' If pData is a NULL-pointer the resource manager is told that the resource doesnt have any data 
' (e.g. the corresponding file was not found).
' In this case, the resource remains in the unloaded state but is no more returned when querying unloaded resources.
' When the specified resource is already loaded, the function returns false.
declare function h3dLoadResource(byval res as TH3DRes,byval pData as const any ptr, byval size as long) as Boolean

' This function unloads a previously loaded resource and restores the default values it had before loading.
' The state is set back to unloaded which makes it possible to load the resource again.
declare sub      h3dUnloadResource(byval res as TH3DRes)

' This function returns how many elements of the specified element type a specified resource has.
declare function h3dGetResElemCount(byval res as TH3DRes ,byval ElemType as long) as long

' This function searches in a specified resource for the first element of the specified type 
' that has the property with the specified name set to the specified search value.
' If such element is found, its index is returned, otherwise the function returns -1.
' All string comparisons done for the search are case-sensitive.
declare function h3dFindResElem(byval res as TH3DRes, byval elem as long, byval param as long, byval value as const zstring ptr) as long

' This function set/get a specified property of the specified resource element.
' The property must be of the type int.
declare function h3dGetResParamI(byval res as TH3DRes, byval elem as long, byval elemIdx as long, byval param as long) as long
declare sub      h3dSetResParamI(byval res as TH3DRes, byval elem as long, byval elemIdx as long, byval param as long, byval value as long) 

' This function sets/gets the specified component of the specified resource element property.
' The property must be of the type float. Float properties can have several indexed components, for example the x, y and z components of a vector.
declare function h3dGetResParamF(byval res as TH3DRes, byval elem as long, byval elemIdx as long, byval param as long, byval compIdx as long) as single
declare sub      h3dSetResParamF(byval res as TH3DRes, byval elem as long, byval elemIdx as long, byval param as long, byval compIdx as long, byval value as Single)

' This function set/get a specified property of the specified resource element.
' The property must be of the type string.
' Important Note: The pointer is const and allows only read access to the data.
' Do never try to modify the data of the pointer since that can corrupt the engines internal states!
declare function h3dGetResParamStr(byval res as TH3DRes, byval elem as long, byval elemIdx as long, byval param as long) as const zstring ptr 
declare sub      h3dSetResParamStr(byval res as TH3DRes, byval elem as long, byval elemIdx as long, byval param as long, byval value as const zstring ptr) 

' This function maps the specified stream of a specified resource element and returns a pointer to the stream data.
' The required access to the data can be specified with the read write parameters.
' If read is false, the pointer will usually not contain meaningful data.
' Not all resource streams can be mapped with both read and write access.
' If it is not possible to map the stream, the function will return a NULL pointer.
' A mapped stream should be unmapped again as soon as possible but always before subsequent API calls are made.
' It is only possible to map one stream per resource at a time.
declare function h3dMapResStream(byval res as TH3DRes, byval elem as long, byval elemIdx as long, byval stream as long, _
                                 byval blnRead as Boolean, byval blnWrite as Boolean) as any pointer
' This function unmaps a resource stream that has been mapped before.
declare sub      h3dUnmapResStream(byval res as TH3DRes)

' This function looks for a resource that is not yet loaded and returns its handle.
' If there are no unloaded resources or the zero based index specified 
' is greater than the number of the currently unloaded resources, 0 is returned.
declare function h3dQueryUnloadedResource(byval index as long) as TH3DRes

' This function releases resources that are no longer used.
' Unused resources were either told to be released by the user calling removeResource 
' or are no more referenced by any other engine objects.
declare sub      h3dReleaseUnusedResources()

' --- Specific resource management functions ---

' This function tries to create and add a Texture resource with the specified name to the resource manager.
' If a Texture resource with the same name is already existing, the function fails.
' The texture is initialized with the specified dimensions and the resource is declared as loaded.
' This function is especially useful to create dynamic textures (e.g. for displaying videos) or output buffers for render-to-texture.
declare function h3dCreateTexture(byval resName as const zstring ptr, byval texWidth as long, byval texHeight as long, _
                                  byval texFormat as TH3DFormats, byval textFlags as TH3DResFlags) as TH3DRes


' This function defines a header that is inserted at the beginning of all shaders.
' The preamble is used when a shader is compiled, so changing it will not affect any shaders that are already compiled.
' The preamble is useful for setting platform-specific defines that can be employed 
' for creating several shader code paths, e.g. for supporting different hardware capabilities.
declare sub      h3dSetShaderPreambles(byval vertPreamble as const zstring ptr, byval fragPreamble as const zstring ptr)

' This function sets the specified shader uniform of the specified material to the specified values.
declare function h3dSetMaterialUniform(materialRes as TH3DRes, byval UniformName as const zstring ptr, _
                                       byval a as single, byval b as single=0, byval c as single=0, byval d as single=0) as Boolean

' This function sets the base width and height which affects render targets with relative (in percent) size specification.
' Changing the base size is usually desired after engine initialization and when the window is being resized.
' Note that in case several cameras use the same pipeline resource instance, the change will affect all cameras.
declare sub      h3dResizePipelineBuffers(byval pipeRes as TH3DRes, byval bufWidth as long, byval bufHeight as long)

' This function reads back the pixels of a specified render target.
' If no pipeline resource is specified, the color data of the backbuffer is read back as RGBA float.
' Otherwise, the specified buffer of the specified render target is read back and stored in dataBuffer as RGBA float values.
' To compute the required size of the output buffer to which the data is written, the function can be called 
' with a NULL pointer for dataBuffer and pointers to variables where the buffer width, height and the number of components will be stored.  
' As this function has a considerable performance overhead, it is only intended for debugging purposes and screenshots.
' For more information about the render buffers, refer to the Pipeline documentation.
declare function h3dGetRenderTargetData(byval pipelineRes as TH3DRes, _
                                        byval targetName  as const zstring ptr, _
                                        byval bufIndex    as long , _
                                        byval bufWidth    as long ptr , _
                                        byval bufHeight   as long ptr , _
                                        byval compCount   as long ptr , _
                                        byval dataBuffer  as any  ptr , _
                                        byval bufSize     as long) as Boolean
#define h3dGetRenderTargetSize(w32,h32) h3dGetRenderTargetData(0,"",0,@w32,@h32,0,0,0)
' --- General scene graph functions ---

' This function returns the type of a specified scene node.
' If the node handle is invalid, the function returns the node type 'H3DNodeTypes_Undefined'.
declare function h3dGetNodeType(byval node as TH3DNode) as TH3DNodeTypes

' This function returns the handle to the parent node of a specified scene node.
' If the specified node handle is invalid or the root node, 0 is returned.
declare function h3dGetNodeParent(byval node as TH3DNode) as TH3DNode 

' This function relocates a scene node.  
' It detaches the node from its current parent and attaches it to the specified new parent node.
' If the attachment to the new parent is not possible, the function returns false.
' Relocation is not possible for the H3DRootNode.
declare function h3dSetNodeParent(byval node as TH3DNode,byval parent as TH3DNode) as Boolean

' This function looks for the n-th (index) child node of a specified node and returns its handle.
' If the child doesnt exist, the function returns 0.
declare function h3dGetNodeChild(byval node as TH3DNode,byval index as long) as TH3DNode

' This function creates several new nodes as described in a SceneGraph resource and attaches them to a specified parent node.
' If an invalid scenegraph resource is specified or the scenegraph resource is unloaded, the function returns 0.
declare function h3dAddNodes(byval parent as TH3DNode,byval sceneGraphRes as TH3DRes) as TH3DNode

' This function removes the specified node and all of its children from the scene.
declare sub      h3dRemoveNode(byval node as TH3DNode)

' This function checks if a scene node has been transformed by the engine since the last time the transformation flag was reset.
' Therefore, it stores a flag that is set to true when a setTransformation function is called explicitely 
' by the application or when the node transformation has been updated by the animation system.
' The function also makes it possible to reset the transformation flag.
declare function h3dCheckNodeTransFlag(byval node as TH3DNode, byval resetFlag as Boolean) as Boolean

' This function get/set the translation, rotation and scale of a specified scene node object.
' The coordinates are in local space and contain the transformation of the node relative to its parent.
declare sub      h3dGetNodeTransform(byval node as TH3DNode, _
                                     byval tx as single ptr, byval ty as single ptr, byval tz as single ptr, _
                                     byval rx as single ptr, byval ry as single ptr, byval rz as single ptr, _
                                     byval sx as single ptr, byval sy as single ptr, byval sz as single ptr)

declare sub      h3dSetNodeTransform(byval node as TH3DNode, _
                                     byval tx as single  , byval ty as single  , byval tz as single, _
                                     byval rx as single=0, byval ry as single=0, byval rz as single=0, _
                                     byval sx as single=1, byval sy as single=1, byval sz as single=1)

' This function stores a pointer to the relative and absolute transformation matrices of the specified node in the specified pointer varaibles.
' (relMat or absMat can be NULL if matrix is not required)
declare sub      h3dGetNodeTransMats(byval node as TH3DNode,byval relMat as const single ptr ptr, byval absMat as const single ptr ptr) 
' This function sets the relative transformation matrix of the specified scene node.
' It is basically the same as setNodeTransform but takes directly a matrix instead of individual transformation parameters.
declare sub      h3dSetNodeTransMat(byval node as TH3DNode, byval mat4x4 as const single ptr)

' This function gets/sets a specified property of the specified node.
declare function h3dGetNodeParamI(byval node as TH3DNode,byval param as long) as long
declare sub      h3dSetNodeParamI(byval node as TH3DNode,byval param as long,byval value as long) 

' This function gets/sets a specified property of the specified node.
' Float properties can have several indexed components, for example the x, y and z components of a vector.
declare function h3dGetNodeParamF(byval node as TH3DNode,byval param as long,byval compIdx as long) as single
declare sub      h3dSetNodeParamF(byval node as TH3DNode,byval param as long,byval compIdx as long,byval value as single)

' This function returns a specified property of the specified node.
' The property must be of the type string.
' Important Note: The pointer is const and allows only read access to the data.
' Do never try to modify the data of the pointer since that can corrupt the engines internal states!
declare function h3dGetNodeParamStr(byval node as TH3DNode,byval param as TH3DNodeParams) as const zstring ptr 

' This function sets a specified property of the specified node to a specified value.
declare sub      h3dSetNodeParamStr(byval node as TH3DNode,byval param as TH3DNodeParams,byval value as const zstring ptr) 

' This function returns a bit mask containing the set scene node flags.
declare function h3dGetNodeFlags(byval node as TH3DNode) as TH3DNodeFlags

' This function sets the flags of the specified scene node.
' flagRecursive specifies whether flags should be applied recursively to all child nodes
declare sub      h3dSetNodeFlags(byval node as TH3DNode,byval flags as TH3DNodeFlags,byval Recursive as Boolean) 

' This function stores the world coordinates of the axis aligned bounding box of a specified node in the specified variables.
' The bounding box is represented using the minimum and maximum coordinates on all three axes.
declare sub      h3dGetNodeAABB(byval node as TH3DNode, _
                                byval minX as single ptr,byval minY as single ptr,byval minZ as single ptr, _
                                byval maxX as single ptr,byval maxY as single ptr,byval maxZ as single ptr)

' This function loops recursively over all children of startNode 
' and adds them to an internal list of results if they match the specified name and type.
' The result list is cleared each time this function is called.
' The function returns the number of nodes which were found and added to the list.
' parameter name  = name of nodes to be searched (empty string for all nodes)
' parameter type  = type of nodes to be searched (H3DNodeTypes_Undefined for all types)
declare function h3dFindNodes(byval startNode as TH3DNode, _
                              byval nodeName as const zstring ptr,_
                              byval nodeType as TH3DNodeTypes) as long

' This function returns the n-th (index) result of a previous h3dFindNodes query.
' The result is the handle to a scene node with the poperties specified at the findNodes query.
' If the index doesnt exist in the result list the function returns 0.
declare function h3dGetNodeFindResult(byval index as long) as TH3DNode

' This function checks recursively if the specified ray intersects the specified node or one of its children.
' The function finds intersections relative to the ray origin and returns the number of intersecting scene nodes.
' The ray is a line segment and is specified by a starting point (the origin) and a finite direction vector which also defines its length.
' Currently this function is limited to returning intersections with Meshes.
' For Meshes, the base LOD (LOD0) is always used for performing the ray-triangle intersection tests.
' Parameter: numNearest = maximum number of intersection points to be stored (0 for all)
declare function h3dCastRay(byval node as TH3DNode, _
                            byval ox as single,byval oy as single,byval oz as single, _
                            byval dx as single,byval dy as single,byval dz as single, _
                            byval numNearest as long) as long

' This functions is used to access the results of a previous h3dCastRay query.
' This function returns the n-th (index) result of a previous h3dCastRay query.
' The intersection data is copied to the specified variables.
' parameter: intersection = coordinates of intersection point array of single (x,y,z)
declare function h3dGetCastRayResult(byval index     as long, _
                                     byval node      as TH3DNode ptr, _
                                     byval fDistance as single ptr, _
                                     byval fVector3  as single ptr) as Boolean

' This function checks if a specified node is visible from the perspective of a specified camera.
' The function always checks if the node is in the cameras frustum.
' If checkOcclusion is true, the function will take into account the occlusion culling information 
' from the previous frame (if occlusion culling is disabled the flag is ignored).
' The flag calcLod determines whether the detail level for the node should be returned in case it is visible.
' The function returns -1 if the node is not visible, otherwise 0 (base LOD level) or the computed LOD level.
declare function h3dCheckNodeVisibility(byval node   as TH3DNode, _
                                        byval camera as TH3DNode, _
                                        byval checkOcclusion as Boolean, _
                                        byval calcLod as Boolean) as long

' --- Group-specific scene graph functions ---

' This function creates a new Group node and attaches it to the specified parent node.
declare function h3dAddGroupNode(byval parent as TH3DNode, byval nodeName as const zstring ptr) as TH3DNode

' --- Model-specific scene graph functions ---

' This function creates a new Model node and attaches it to the specified parent node.
declare function h3dAddModelNode(byval parent as TH3DNode,byval nodeName as const zstring ptr,byval geoRes as TH3DRes) as TH3DNode

' This function is used to setup the specified animation stage/slot of the specified Model node.
' The function can be used for animation blending and mixing.  
' There is a fixed number of stages (by default 16) on which different animations can be played.
' The start node determines the first node (Joint or Mesh) to which the animation is recursively applied.
' If the start node is an empty string, the animation affects all animatable nodes (Joints and Meshes) of the model.
' If a NULL-handle is used for animationRes, the stage is cleared and the currently set animation is removed.
' The layer determines the priority of the animation and how the weights are distributed.  See h3dSetModelAnimParams for more information.
' A simple way to do animation mixing is using additive animations.  
' If a stage is configured to be additive, the engine calculates the difference between the current frame 
' and the first frame in the animation and adds this delta, scaled by the weight factor, to the current transformation of the joints or meshes.  
' Additive animations completely ignore the layer settings.  
' They are only applied by the engine if a non-additive animation is assigned to the model as well.
declare sub      h3dSetupModelAnimStage(byval modelNode as TH3DNode, _
                                        byval animStage as long, _
                                        byval animRes   as TH3DRes, _
                                        byval layer     as long, _
                                        byval startNode as const zstring ptr, _
                                        byval additive  as Boolean) 

' This function sets the current animation time and weight for a specified stage of the specified model.
' The time corresponds to the frames of the animation and the animation is looped 
' if the time is higher than the maximum number of frames in the Animation resource.
' The weight is used for animation blending and determines how much influence the stage has compared to the other active stages.
' All weights of stages that are on the same layer are normalized.
' Stages with a higher layer id are animated before stages with lower layers.
' The blend weight is distributed across the layers.  
' If the weight sum of a layer is smaller than 1.0, the remaining weight is propagated to the next lower layer.
' So if a layer uses a weight of 100%, the lower layers will not get any weight and consequently not contribute to the overall animation.
declare sub      h3dSetModelAnimParams(byval modelNode as TH3DNode,byval stage as long,byval animTime as single,byval weight as single) 

' This function sets the weight of a specified morph target.
' If the target parameter is an empty string the weight of all morph targets in the specified Model node is modified.
' If the specified morph target is not found the function returns false.
declare function h3dSetModelMorpher(byval modelNode as TH3DNode,byval morphTarget as const zstring ptr,byval weight as single) as Boolean

' Function: h3dUpdateModel
'   Applies animation and/or geometry updates.
'  Details:
'    This function applies skeletal animation and geometry updates to the specified model, depending on
'    the specified update flags. Geometry updates include morph targets and software skinning if enabled.
'    If the animation or morpher parameters did not change, the function returns immediately. This function
'    has to be called so that changed animation or morpher parameters will take effect.
'  Parameters:
'    modelNode  - handle to the Model node to be updated
'    flags      - combination of H3DModelUpdate flags
declare sub h3dUpdateModel(byval modelNode as TH3DNode,byval flags as H3DModelUpdateFlags)

' --- Mesh-specific scene graph functions --

' This function creates a new Mesh node and attaches it to the specified parent node.
' Parameters: 
'             handle to parent node to which the new node will be attached
'             name of the node
' materialRes = material resource used by Mesh node' 
' batchStart  = first triangle index of mesh in Geometry resource of parent Model node
' batchCount  = number of triangle indices used for drawing mesh
' vertRStart  = first vertex in Geometry resource of parent Model node
' vertREnd    = last vertex in Geometry resource of parent Model node
declare function h3dAddMeshNode(byval parent as TH3DNode,byval nodeName as const zstring ptr,byval materialRes as TH3DRes, _
                                byval batchStart as long, batchCount as long, vertRStart as long, vertREnd as long) as TH3DNode

' --- Joint-specific scene graph functions ---

' This function creates a new Joint node and attaches it to the specified parent node.
' Parameters:
' parent     = handle to parent node to which the new node will be attached
' name       = name of the node
' jointIndex = index of joint in Geometry resource of parent Model node
declare function h3dAddJointNode(byval parent as TH3DNode, nodeName as const zstring ptr,byval jointIndex as long) as TH3DNode

' --- Light-specific scene graph functions ---

' This function creates a new Light node and attaches it to the specified parent node.
' The direction vector of the untransformed light node is pointing along the the negative z-axis.
' The specified material resource can define uniforms and projective textures.
' Furthermore it can contain a shader for doing lighting calculations if deferred shading is used.
' If no material is required the parameter can be zero.
' The context names define which shader contexts are used when rendering shadow maps or doing light calculations for forward rendering configurations.
' Parameters:
' parent          = handle to parent node to which the new node will be attached
' name            = name of the node
' materialRes     = material resource for light configuration or 0 if not used
' lightingContext = name of the shader context used for doing light calculations
' shadowContext   = name of the shader context used for doing shadow map rendering
declare function h3dAddLightNode(byval parent          as TH3DNode, _
                                 byval nodeName        as const zstring ptr, _
                                 byval materialRes     as TH3DRes, _
                                 byval lightingContext as const zstring ptr, _
                                 byval shadowContext   as const zstring ptr) as TH3DNode

' --- Camera-specific scene graph functions ---

' This function creates a new Camera node and attaches it to the specified parent node.
' Parameters: 
' parent      = handle to parent node to which the new node will be attached
' name        = name of the node
' pipelineRes = pipeline resource used for rendering
declare function h3dAddCameraNode(byval parent as TH3DNode,byval nodeName as const zstring ptr,byval pipelineRes as TH3DRes) as TH3DNode

' This function calculates the view frustum planes of the specified camera node using the specified view parameters.
declare sub      h3dSetupCameraView(byval cameraNode as TH3DNode,byval fov as single,byval aspect as single,byval nearDist as single,byval farDist as single) 

' This function gets the camera projection matrix used for bringing the geometry to screen space and copies it to the specified array.
declare sub      h3dGetCameraProjMat(byval cameraNode as TH3DNode,byval mat4x4 as single ptr)

' --- Emitter-specific scene graph functions ---

' This function creates a new Emitter node and attaches it to the specified parent node.
' Parameters: 
' parent            = handle to parent node to which the new node will be attached
' nodeName          = name of the node
' materialRes       = handle to Material resource used for rendering
' particleEffectRes = handle to ParticleEffect resource used for configuring particle properties
' maxParticleCount  = maximal number of particles living at the same time
' respawnCount      = number of times a single particle is recreated after dying (-1 for infinite)
declare function h3dAddEmitterNode(byval parent            as TH3DNode, _
                                   byval nodeName          as const zstring ptr, _
                                   byval materialRes       as TH3DRes, _
                                   byval particleEffectRes as TH3DRes, _
                                   byval maxParticleCount  as long, _
                                   byval respawnCount      as long=-1) as TH3DNode

' This function advances the simulation time of a particle system and continues the particle simulation 
' with timeDelta being the time elapsed since the last call of this function.
' The specified node must be an Emitter node.
' Advances the time value of an Emitter node.
declare sub      h3dUpdateEmitter(byval emitterNode as TH3DNode, byval timeDelta as single) 

' This function checks if a particle system is still active and has living particles or will spawn new particles.
' The specified node must be an Emitter node.  
' The function can be used to check when a not infinitely running emitter for an effect 
' like an explosion can be removed from the scene.
declare function h3dHasEmitterFinished(byval emitterNode as TH3DNode) as Boolean


' Horde3D Terrain extension

' Function: h3dextAddTerrainNode
' Adds a Terrain node to the scene.

' Details: This function creates a new Terrain node and attaches it to the specified parent node.

' Parameters:
' parent        - handle to parent node to which the new node will be attached
' name          - name of the node
' heightMapRes  - handle to a 2D Texture resource that contains the terrain height information (must be square and POT) 
' materialRes   - handle to the Material resource used for rendering the terrain

' Returns:      handle to the created node or 0 in case of failure
const as long H3DEXT_NodeType_Terrain = 100
declare function h3dextAddTerrainNode(byval parent     as TH3DNode, _
                                      byval nodeName   as const zstring ptr, _
                                      byval heightMap  as TH3DRes, _
                                      byval terrainMat as TH3DRes) as TH3DNode

' Function: h3dextCreateTerrainGeoRes
' Creates a Geometry resource from a specified Terrain node.

' Details:
' This function creates a new Geometry resource that contains the vertex data of the specified Terrain node.
' To reduce the amount of data, it is possible to specify a quality value which controls the overall resolution
' of the terrain mesh. The algorithm will automatically create a higher resoultion in regions where the
' geometrical complexity is higher and optimize the vertex count for flat regions.

' Parameters:
' node         - handle to terrain node that will be accessed
' resName      - name of the Geometry resource that shall be created
' meshQuality  - constant controlling the overall mesh resolution

' Returns:     handle to the created Geometry resource or 0 in case of failure
declare function h3dextCreateTerrainGeoRes(byval node        as TH3DNode, _
                                           byval resName     as const zstring ptr, _
                                           byval meshQuality as single) as TH3DRes

' Some words about the Utility Library.
' Horde3D has a simple core API which offers all the functionality needed to control the engine.
' The engine core is intended to be as generic as possible to make the complete system lightweight and clean.
' Nevertheless, it is sometimes useful to have more specific functions in order to increase productivity.
' For this reason the Utility Library is offered. 
' It has some very handy functions that can help to make your life easier.

' Group: Typedefs and constants 
' Maximum stat mode number supported in showFrameStats
const as long H3DUTMaxStatMode = 2

' --- General functions ---

' Frees memory allocated by the Utility Library.
' This utility function frees the memory that was allocated by another function of the Utility Library.
' Parameters:  ptr address of a pointer that references to memory allocated by the Utility Library
declare sub      h3dutFreeMem(byref p as any ptr)

' Writes all messages in the queue to a log file.
' This utility function pops all messages from the message queue and writes them to a HTML formated log file Horde3D_Log.html.
' Returns   : true in case of success, otherwise false
declare function h3dutDumpMessages() as Boolean
declare sub      h3dutSetDumpFile(byval filename as const zstring ptr)
declare sub      h3dutSetDumpTitle(byval title   as const zstring ptr)


#ifdef __FB_WIN32__
' --- OpenGL-related functions ---
' Initializes OpenGL.
' This utility function initializes an OpenGL rendering context in a specified window component.
' Currently this function is only available on Windows platforms.
' Parameters: hDC handle to device context for which OpenGL context shall be created
' Returns:    true in case of success, otherwise false
declare function h3dutInitOpenGL(byval hDC as any ptr) as Boolean

' Releases OpenGL.
' This utility function destroys the previously created OpenGL rendering context.
' Currently this function is only available on Windows platforms.
declare sub      h3dutReleaseOpenGL()

' Displays the rendered image on the screen.
' This utility function displays the image rendered to the previously initialized OpenGL context 
' on the screen by copying it from the backbuffer to the frontbuffer.
' Currently this function is only available on Windows platforms.
declare sub      h3dutSwapBuffers()
#endif

' --- Resource management ---
' Deprecated Returns the search path of a resource type.
' This function returns the search path of a specified resource type.
' The function is now marked as deprecated since it is better practice to make all paths relative to the content directory.
' Parameters: type of resource
' Returns: pointer to the search path string
declare function h3dutGetResourcePath(byval resType as TH3DResTypes) as const zstring ptr

' Deprecated Sets the search path for a resource type.
' This function sets the search path for a specified resource type.
' The function is now marked as deprecated since it is better practice to make all paths relative to the content directory.
' Parameters: type of resource
'             path where the resources can be found ((back-)slashes at end are removed)
declare sub      h3dutSetResourcePath(byval resType as TH3DResTypes, byval path as const zstring ptr)

' Loads previously added resources from a data drive.
' This utility function loads previously added and still unloaded resources from the specified directories on a data drive.  
' Several search paths can be specified using the pipe character "|" as separator.
' All resource names are directly converted to filenames and the function tries to find them 
' in the specified directories using the given order of the search paths.
' Parameters: directories where data is located on the drive ((back-)slashes at end are removed)
' Returns:    false if at least one resource could not be loaded, otherwise true
declare function h3dutLoadResourcesFromDisk(byval contentDir as zstring ptr) as Boolean

' Creates a Geometry resource from specified vertex data.
' This utility function allocates and initializes a Geometry resource with the specified vertex attributes and indices.
' The optional tangent space data (normal, tangent, bitangent) is encoded as short, where -1.0 maps to -32767 and 1.0 to +32767.
declare function h3dutCreateGeometryRes(byval resName as zstring ptr, _
                                        byval numVertices as long, byval numTriangleIndices as long, _
                                        byval posData    as single ptr, byval indexData   as ulong ptr, _
                                        byval normalData as short  ptr, byval tangentData as short ptr, byval bitangentData as short ptr, _
                                        byval texData1   as single ptr, byval texData2    as single ptr) as TH3DRes

' Creates a TGA image in memory.
' This utility function allocates memory at the pointer outData and creates a TGA image from the specified pixel data.
' The dimensions of the image have to be specified as well as the bit depth.  
' The created TGA-image-data can be used as Texture2D or TexureCube resource in the engine.
' Note: The memory allocated by this routine has to freed manually using the h3dutFreeMem function.
declare function h3dutCreateTGAImage(byval pixels as const any ptr, _
                                     byval texWidth as long, texHeight as long, _
                                     byval depth as long, byref outData as any ptr, byval outSize as long ptr) as Boolean

' --- Utils ---
' Writes the content of the backbuffer to a tga file.
' This function reads back the content of the backbuffer and writes it to a tga file with the specified filename and path.
' Parameters: filename and path of the output tga file
' Returns:    true if the file could be written, otherwise false
declare function h3dutScreenshot(byval fileName as const zstring ptr) as Boolean



' --- Scene graph ---

declare sub h3dutLookAt(byval camera as TH3DNode, _
                        byval px as single, byval py as single, byval pz as single, _
                        byval tx as single, byval ty as single, byval tz as single, _
                        byval ux as single, byval uy as single, byval uz as single)

' Calculates the ray originating at the specified camera and window coordinates
' This utility function takes normalized window coordinates 
' (ranging from 0 to 1 with the origin being the bottom left corner of the window) 
' and returns ray origin and direction for the given camera.  
' The function is especially useful for selecting objects by clicking on them.
' Parameters: camera used for picking
'             nwx, nwy normalized window coordinates
'             ox, oy, oz calculated ray origin
'             dx, dy, dz calculated ray direction
declare sub      h3dutPickRay(byval camera as TH3DNode, _
                              byval nwx as single    ,byval nwy as single, _
                              byval ox  as single ptr,byval oy  as single ptr,byval oz as single ptr, _
                              byval dx  as single ptr,byval dy  as single ptr,byval dz as single ptr)

' Returns the scene node which is at the specified window coordinates.
' This utility function takes normalized window coordinates 
' (ranging from 0 to 1 with the origin being the bottom left corner of the window) 
' and returns the scene node which is visible at that location.  
' The function is especially useful for selecting objects by clicking on them. 
' Currently picking is only working for Meshes.
' Parameters: camera used for picking
'              nwx, nwy normalized window coordinates
' Returns:    handle of picked node or 0 if no node was hit
declare function h3dutPickNode(byval camera as TH3DNode,byval nwx as single,byval nwy as single) as TH3DNode

' --- Overlays ---
' Shows text on the screen using a font texture.
' This utility function uses overlays to display a text string at a specified position on the screen.
' The font texture of the specified font material has to be a regular 16x16 grid containing all ASCII characters in row-major order.
' Parameters: text string to be displayed
'             position of the lower left corner of the first character
'             size (scale) factor of the font
'             font color
'             font material resource used for rendering
declare sub      h3dutShowText(byval text as zstring ptr, _
                               byval x as single, byval y as single, byval size as single, _
                               byval r as single, byval g as single, byval b as single, _
                               byval fontMat as TH3DRes)

' Shows frame statistics on the screen.
' This utility function displays an info box with statistics for the current frame on the screen.  
' Since the statistic counters are reset after the call, the function should be called exactly once per frame to obtain correct values.
' Parameters: font material resource used for drawing text 
'             material resource used for drawing info box
'             display mode, specifying which data is shown (<= MaxStatMode)
declare sub      h3dutShowFrameStats(byval fontMat as TH3DRes,byval boxMat as TH3DRes,byval mode as long)


declare sub      h3dutShowInfoBox(byval x        as single, _
                                  byval y        as single, _
                                  byval w        as single, _
                                  byval title    as const zstring ptr, _
                                  byval nRows    as integer, _
                                  byval column1  as const zstring ptr ptr, _
                                  byval column2  as const zstring ptr ptr, _
                                  byval fontMat  as TH3DRes, _
                                  byval panelMat as TH3DRes)
end extern

#endif ' __horde3d_bi__
