#include once "fbgfx.bi"
#include once "horde3d.bi"

' load a manualy creates geo, scene and material file.

function InitHorde3D(byval w as integer=640,byval h as integer=480) as boolean
  dim as integer bits
  screeninfo ,,bits
  ' create window with OpenGL context
  screenres w,h,bits,,fb.GFX_OPENGL
  if screenptr()=0 then
    screen 0
    print "error: init OpenGL context !"
    beep : sleep : end 1
  end if
  flip
  ' init Horde3D
  var ret = h3dInit()
  if ret=false then h3dutDumpMessages()
  return ret
end function

if InitHorde3D()=false then
  h3dutDumpMessages()
  screen 0
  print "error: init Horde3D !"
  beep : sleep : end 1
end if
' get current screen size
dim as integer w,h
screeninfo w,h

dim as string dumpFile  = command(0) & ".html"
dim as string dumpTitle = "load cube.geo with: " & *h3dGetVersionString

h3dutSetDumpFile (strptr(dumpFile))
h3dutSetDumpTitle(strptr(dumpTitle))

' Add resources
var renderPipeline = h3dAddPipeline  ("pipelines/forward.pipeline.xml")
var platformScene  = h3dAddSceneGraph("models/platform/platform.scene.xml")
var cubeScene      = h3dAddSceneGraph("models/cube/cube.scene.xml")

' Load resources
var contentDir = exepath() + "/Content"
h3dutLoadResourcesFromDisk(strptr(contentDir))

' optional dump log messages
h3dutDumpMessages() ' usefull if loading of any resources are fails

var platformNode = h3dAddNodes(H3DRootNode, platformScene)
var cubeNode     = h3dAddNodes(H3DRootNode, cubeScene)
var camNode      = h3dAddCameraNode(H3DRootNode, "Camera", RenderPipeline)
h3dSetNodeTransform(camNode, 0,5,10, -30,0,0, 1,1,1)

' Setup size of the viewport
h3dSetNodeParamI(camNode, H3DCamera_ViewportWidthI , w)
h3dSetNodeParamI(camNode, H3DCamera_ViewportHeightI, h)

' Set camera parameters (field of view, ratio, near plane, far plane)
h3dSetupCameraView(camNode, 45.0, w/h, 0.1, 1000.0)

' Setup size of the render target
h3dResizePipelineBuffers(RenderPipeline, w,h)

' Add light source
var lightNode = h3dAddLightNode(H3DRootNode, "Light1", 0, "LIGHTING", "SHADOWMAP")
h3dSetNodeTransform(lightNode, 0,15,0, 90*3,0,0, 1,1,1)
h3dSetNodeParamI(lightNode, H3DLight_ShadowMapCountI, 1)


' shows the bounding boxes
'h3dSetOption(H3DOptions_DebugViewMode,1)
' usefull if something are wrong with trriangles
'h3dSetOption(H3DOptions_WireframeMode,1)
dim as single xRot,yRot,zRot
while inkey()=""
  ' transform the cube
  h3dSetNodeTransform(cubeNode,0,2,0, xRot,yRot,zRot, 2,2,2)

  ' Render scene
  h3dRender(camNode)
  ' Finish rendering of frame
  h3dFinalizeFrame()
  ' swap buffers
  flip

  xRot += .3
  yRot += .6
  zRot += .9
wend

' free all resources
h3dRelease()
