const as ulong DDS_MAGIC = &H20534444 ' "DDS "

type DDS_HEADER_FLAGS as ulong
const as DDS_HEADER_FLAGS DDSD_CAPS        = &H000001 ' Required in every .dds file.
const as DDS_HEADER_FLAGS DDSD_HEIGHT      = &H000002 ' Required in every .dds file.
const as DDS_HEADER_FLAGS DDSD_WIDTH       = &H000004 ' Required in every .dds file.
const as DDS_HEADER_FLAGS DDSD_PIXELFORMAT = &H001000 ' Required in every .dds file.

const as DDS_HEADER_FLAGS DDSD_PITCH       = &H000008 ' Required when pitch is provided for an uncompressed texture.
const as DDS_HEADER_FLAGS DDSD_LINEARSIZE  = &H080000 ' Required when pitch is provided for a compressed texture.
const as DDS_HEADER_FLAGS DDSD_MIPMAPCOUNT = &H020000 ' Required in a mipmapped texture.
const as DDS_HEADER_FLAGS DDSD_DEPTH       = &H800000 ' Required in a depth texture.

' Note  When you write .dds files, you should set the DDSD_CAPS and DDSD_PIXELFORMAT flags, 
' and for mipmapped textures you should also set the DDSD_MIPMAPCOUNT flag. 
' However, when you read a .dds file, you should not rely on the DDSD_CAPS, DDSD_PIXELFORMAT, and DDSD_MIPMAPCOUNT flags being set 
' because some writers of such a file might not set these flags.

' defined in Dds.h
const as DDS_HEADER_FLAGS DDS_HEADER_FLAGS_TEXTURE    = DDSD_CAPS or DDSD_HEIGHT or DDSD_WIDTH or DDSD_PIXELFORMAT
const as DDS_HEADER_FLAGS DDS_HEADER_FLAGS_MIPMAP     = DDSD_MIPMAPCOUNT
const as DDS_HEADER_FLAGS DDS_HEADER_FLAGS_VOLUME     = DDSD_DEPTH
const as DDS_HEADER_FLAGS DDS_HEADER_FLAGS_PITCH      = DDSD_PITCH
const as DDS_HEADER_FLAGS DDS_HEADER_FLAGS_LINEARSIZE = DDSD_LINEARSIZE

type DDS_PIXELFORMAT_FLAGS as ulong
const as DDS_PIXELFORMAT_FLAGS DDPF_ALPHAPIXELS = &H00001 ' Texture contains alpha data; dwRGBAlphaBitMask contains valid data.
const as DDS_PIXELFORMAT_FLAGS DDPF_ALPHA       = &H00002 ' old for alpha channel only uncompressed data (dwRGBBitCount contains the alpha channel bitcount; dwABitMask contains valid data)
const as DDS_PIXELFORMAT_FLAGS DDPF_FOURCC      = &H00004 ' Texture contains compressed RGB data; dwFourCC contains valid data.
const as DDS_PIXELFORMAT_FLAGS DDPF_RGB         = &H00040 ' Texture contains uncompressed RGB data; dwRGBBitCount and the RGB masks (dwRBitMask, dwGBitMask, dwBBitMask) contain valid data.
const as DDS_PIXELFORMAT_FLAGS DDPF_YUV         = &H00200 ' old for YUV uncompressed data (dwRGBBitCount contains the YUV bit count; dwRBitMask contains the Y mask, dwGBitMask contains the U mask, dwBBitMask contains the V mask)
const as DDS_PIXELFORMAT_FLAGS DDPF_LUMINANCE   = &H20000 ' old for single channel color uncompressed data (dwRGBBitCount contains the luminance channel bit count; dwRBitMask contains the channel mask).
                                                          ' Can be combined with DDPF_ALPHAPIXELS for a two channel DDS file.

type DDS_PIXELFORMAT
  as ulong dwSize                         ' Structure size; set to 32 (bytes).
  as ulong dwFlags                        ' Values which indicate what type of data is in the surface. 
  as ulong dwFourCC                       ' Possible values include: DXT1, DXT2, DXT3, DXT4, or DXT5. A FourCC of DX10 indicates the prescense of the DDS_HEADER_DXT10 extended header, 
                                          ' and the dxgiFormat member of that structure indicates the true format.
                                          ' When using a four-character code, dwFlags must include DDPF_FOURCC.
  as ulong dwRGBBitCount                  ' Number of bits in an RGB (possibly including alpha) format. Valid when dwFlags includes DDPF_RGB, DDPF_LUMINANCE, or DDPF_YUV.
  as ulong dwRBitMask                     ' Red (or lumiannce or Y) mask for reading color data. For instance, given the A8R8G8B8 format, the red mask would be 0x00ff0000.
  as ulong dwGBitMask                     ' Green (or U) mask for reading color data. For instance, given the A8R8G8B8 format, the green mask would be 0x0000ff00.
  as ulong dwBBitMask                     ' Blue (or V) mask for reading color data. For instance, given the A8R8G8B8 format, the blue mask would be 0x000000ff.
  as ulong dwABitMask                     ' dwFlags must include DDPF_ALPHAPIXELS or DDPF_ALPHA, given the A8R8G8B8 format, the alpha mask would be 0xff000000.
end type

type DDS_HEADER_CAPS as ulong
const as DDS_HEADER_CAPS DDSCAPS_COMPLEX = &H000008 ' Optional; must be used on any file that contains more than one surface (a mipmap, a cubic environment map, or mipmapped volume texture).
const as DDS_HEADER_CAPS DDSCAPS_MIPMAP  = &H400000 ' Optional; should be used for a mipmap.
const as DDS_HEADER_CAPS DDSCAPS_TEXTURE = &H001000 '  Required
' defined in Dds.h
const as DDS_HEADER_CAPS DDS_SURFACE_FLAGS_MIPMAP  = DDSCAPS_COMPLEX or DDSCAPS_MIPMAP
const as DDS_HEADER_CAPS DDS_SURFACE_FLAGS_TEXTURE = DDSCAPS_TEXTURE
const as DDS_HEADER_CAPS DDS_SURFACE_FLAGS_CUBEMAP = DDSCAPS_COMPLEX

type DDS_HEADER_CAPS2 as ulong
const as DDS_HEADER_CAPS2 DDSCAPS2_CUBEMAP           = &H000200 ' Required for a cube map.
const as DDS_HEADER_CAPS2 DDSCAPS2_CUBEMAP_POSITIVEX = &H000400 ' Required when these surfaces are stored in a cube map.
const as DDS_HEADER_CAPS2 DDSCAPS2_CUBEMAP_NEGATIVEX = &H000800 ' Required when these surfaces are stored in a cube map.
const as DDS_HEADER_CAPS2 DDSCAPS2_CUBEMAP_POSITIVEY = &H001000 ' Required when these surfaces are stored in a cube map.
const as DDS_HEADER_CAPS2 DDSCAPS2_CUBEMAP_NEGATIVEY = &H002000 ' Required when these surfaces are stored in a cube map.
const as DDS_HEADER_CAPS2 DDSCAPS2_CUBEMAP_POSITIVEZ = &H004000 ' Required when these surfaces are stored in a cube map.
const as DDS_HEADER_CAPS2 DDSCAPS2_CUBEMAP_NEGATIVEZ = &H008000 ' Required when these surfaces are stored in a cube map.
const as DDS_HEADER_CAPS2 DDSCAPS2_VOLUME            = &H200000 ' Required for a volume texture.
' defined in Dds.h
const as DDS_HEADER_CAPS2 DDS_CUBEMAP_ALLFACES = DDSCAPS2_CUBEMAP _
                                              or DDSCAPS2_CUBEMAP_POSITIVEX _
                                              or DDSCAPS2_CUBEMAP_NEGATIVEX _
                                              or DDSCAPS2_CUBEMAP_POSITIVEY _
                                              or DDSCAPS2_CUBEMAP_NEGATIVEY _
                                              or DDSCAPS2_CUBEMAP_POSITIVEZ _
                                              or DDSCAPS2_CUBEMAP_NEGATIVEZ


type DDS_HEADER
  as ulong            dwSize              ' Size of structure. This member must be set to 124.
  as DDS_HEADER_FLAGS dwFlags             ' Flags to indicate which members contain valid data. 
  as ulong            dwHeight            ' Surface height (in pixels).
  as ulong            dwWidth             ' Surface width (in pixels).
  as ulong            dwPitchOrLinearSize ' The scanline pitch in an uncompressed texture or the total number of bytes of compressed top level texture.
  as ulong            dwDepth             ' Depth of a volume texture (in pixels), otherwise unused. 
  as ulong            dwMipMapCount       ' Number of mipmap levels, otherwise unused.
  as ulong            dwReserved1(11-1)   ' Unused.
  as DDS_PIXELFORMAT  ddspf               ' The pixel format 
  as DDS_HEADER_CAPS  dwCaps              ' Specifies the complexity of the surfaces stored.
  as DDS_HEADER_CAPS2 dwCaps2             ' Unused.
  as ulong            dwCaps3             ' Unused.
  as ulong            dwCaps4             ' Unused.
  as ulong            dwReserved2         ' Unused.
end type

' DDS_FILE
'  as ulong      dwMagic             '   4 bytes must be DDS_MAGIC = &H20534444 = "DDS "
'  as DDS_HEADER hdr                 ' 124 bytes
' ... array of bytes that contains the main surface data.
' ... array of bytes that contains the remaining surfaces such as; mipmap levels, faces in a cube map, depths in a volume texture.  


const ambientMap = "../Content/textures/ambientMap.dds"
const skybox     = "../Content/models/skybox/skybox.dds"

dim as boolean IsMipmapped,IsDepth,IsCompressed,IsYUV,IsLuminace,HasAlpha,HasPitch,HasLinareSize
chdir exepath()

var filename = skybox
var hFile = FreeFile
if open(filename,for binary,access read,as #hFile) then
  print "error: can't read'" & filename & " !"
  beep : sleep : end 1
end if
dim as ulong u32
get #hFile,,u32 : 
if u32=DDS_MAGIC then
  dim as DDS_HEADER hdr
  get #hFile,,hdr
  if hdr.dwSize=124 then
    if (hdr.dwFlags and DDS_HEADER_FLAGS_TEXTURE) = DDS_HEADER_FLAGS_TEXTURE then
      print "dwFlags: 0x" & hex(hdr.dwFlags)
      if hdr.dwFlags and DDSD_PITCH       then HasPitch=true     : print "has pitch (uncompressd)"
      if hdr.dwFlags and DDSD_LINEARSIZE  then HasLinareSize=true : IsCompressed=true : print "has linear size (compressed)"
      if hdr.dwFlags and DDSD_MIPMAPCOUNT then IsMipmapped =true : print "mipmapped texture"
      if hdr.dwFlags and DDSD_DEPTH       then IsDepth     =true : print "depth texture"
      print "resolution: " & hdr.dwWidth & " x " & hdr.dwHeight 
      if hdr.dwPitchOrLinearSize then
        print "pitch or linaresize: " & hdr.dwPitchOrLinearSize
        if HasPitch=false andalso HasLinareSize=false then print "warning: has pitch or linaresize!" 
      end if
      if hdr.dwDepth then
        print "depth: " & hdr.dwDepth
        if IsDepth=false then IsDepth=true : print "warning: is depth texture !"
      end if
      if hdr.dwMipMapCount then
        print "number mipmaps: " & hdr.dwMipMapCount
        if IsMipmapped=false then IsMipmapped=true : print "warning: is mipmapped texture !"
      end if
      if hdr.ddspf.dwSize=32 then
        print "pixelformat flags: 0x" & hex(hdr.ddspf.dwFlags)
        if hdr.ddspf.dwFlags and DDPF_ALPHAPIXELS then HasAlpha    = true  : print "has Alpha"
        if hdr.ddspf.dwFlags and DDPF_ALPHA       then HasAlpha    = true  : print "has Alpha (old)"
        if hdr.ddspf.dwFlags and DDPF_FOURCC      then IsCompressed= true  : print "compressed RGB"
        if hdr.ddspf.dwFlags and DDPF_RGB         then IsCompressed= false : print "uncompressed RGB"
        if hdr.ddspf.dwFlags and DDPF_YUV         then IsYUV       = true  : print "compressed YUV"
        if hdr.ddspf.dwFlags and DDPF_LUMINANCE   then IsLuminace  = true  : print "single luminace channel"
        
        if hdr.ddspf.dwFourCC then
          print "FourCC: 0x" & hex(hdr.ddspf.dwFourCC)
          for i as integer = 0 to 3
             print chr((hdr.ddspf.dwFourCC shr (i*8)) and &HFF);
          next
          print
        end if
        if hdr.dwCAPS then
          print "dwCAPS: 0x" & hex(hdr.dwCAPS)
          print  iif(hdr.dwCAPS and DDSCAPS_COMPLEX,"complex ","") & iif(hdr.dwCAPS and DDSCAPS_MIPMAP,"mipmaps ","") & iif(hdr.dwCAPS and DDSCAPS_TEXTURE,"texture","")
        end if
        if hdr.dwCAPS2 then
          print "dwCAPS2: 0x" & hex(hdr.dwCAPS2)
          if hdr.dwCAPS2 and DDSCAPS2_CUBEMAP           then print "CUBEMAP"
          if hdr.dwCAPS2 and DDSCAPS2_CUBEMAP_POSITIVEX then print "POSITIVEX"
          if hdr.dwCAPS2 and DDSCAPS2_CUBEMAP_NEGATIVEX then print "NEGATIVEX"
          if hdr.dwCAPS2 and DDSCAPS2_CUBEMAP_POSITIVEY then print "POSITIVEY"
          if hdr.dwCAPS2 and DDSCAPS2_CUBEMAP_NEGATIVEY then print "NEGATIVEY"
          if hdr.dwCAPS2 and DDSCAPS2_CUBEMAP_POSITIVEZ then print "POSITIVEZ"
          if hdr.dwCAPS2 and DDSCAPS2_CUBEMAP_NEGATIVEZ then print "NEGATIVEZ"
          if hdr.dwCAPS2 and DDSCAPS2_VOLUME            then print "VOLUME"
        end if
      else
        print "error: DDS pixelformat sized is: " & hdr.ddspf.dwSize & " but must be 32 !"
      end if
    else
      print "error: dwFlags are not a rgular DDS texture !"
      beep : sleep
    end if
  else
    print "error: '" & filename & " is a DDS but wrong header size is: " & hdr.dwSize & " but must be 124 !"
    beep : sleep
  end if
else
  print "error: can't read'" & filename & " isn't a regular DDS file !"
  beep : sleep
end if

close #hFile
sleep




