
const as ulong H3DG = &H47443348

type CHUNK_ID as long 
const as CHUNK_ID POSITON_ID         = 0 ' size 12 nVertices * 3 single
const as CHUNK_ID NORMAL_ID          = 1 ' size  6 nVertices * 3 short
const as CHUNK_ID TANGENT_ID         = 2 ' size  6 nVertices * 3 short
const as CHUNK_ID BITANGENTS_ID      = 3 ' size  6 nVertices * 3 short
const as CHUNK_ID JOINT_INDEX_ID     = 4 ' size  4 up to 4 * ubyte (or 0 if unused)
const as CHUNK_ID JOINT_WHEIGHTS_ID  = 5 ' size  4  4 * ubyte 
const as CHUNK_ID TEXTURE_COORDS0_ID = 6 ' size  8 nVertices * 2 single texture coordinates set 0
const as CHUNK_ID TEXTURE_COORDS1_ID = 7 ' size  8 nVertices * 2 single texture coordinates set 1

' Triangle index stream, just after the vertex stream 
#if 0
  as long nTriangleIndicies  '  number of triangle indices n x int

' Morph target stream, just after the triangle index stream 
  as long numMorphTargets    ' can be 0

  ' For each morph target:
  as string*255 morphTargetName  ' 256 chars morph target name, must be null terminated
  as long numMorphVertices       ' number of vertices which differ between base vertices and morph target
  as long morphVertexIndices     ' numMorphVertices of the vertices which should be morphed
  as long numMorphStreams        ' number of parts in morph vertex data. The default implementation uses 4.
                                 ' positions, normals, tangents,bitangents
#endif

type JointInverseBindMatrice
  as single m4x4(15)
end type

type VertexPosition
  as single x,y,z
end type
type VertexNormal
  as short x,y,z  ' single stored as short / 32767.0  
end type
type VertexTangent
  as short x,y,z  ' single stored as short/ 32767.0
end type
type VertexBitangents
  as short x,y,z  ' single stored as short/ 32767.0
end type
type VertexJointIndicies
  as ubyte a,b,c,d ' can be 0=unused
end type
type VertexJointWheights
  as ubyte a,b,c,d ' single stored as byte / 255
end type
type VertexTextureCoords
  as single u,v
end type
type TriangleIndex as long

const sphereFile   = "../Content/models/sphere/sphere.geo"
const cubeFile     = "../Content/models/cube/cube.geo"
const skyboxFile   = "../Content/models/skybox/skybox.geo"
const platformFile = "../Content/models/platform/platform.geo"
const manFile      = "../Content/models/man/man.geo"
const knightFile   = "../Content/models/knight/knight.geo"

chdir exepath

var fileName = platformFile 
var hFile = FreeFile()
if open(fileName,for binary,access read,as #hFile) then
  print "can't read: " & fileName &  "!"
  beep : sleep : end
end if

' check header ID
dim as ulong ID,Version
get #hFile,,ID
if ID<>H3DG then
  close #hFile
  print "no Horde3D geo file !"
  beep : sleep : end
end if

' check version
get #hFile,,version
if version<>5 then
  close #hFile
  print "wrong version " & version & " should be 5 !"
  beep : sleep : end
end if

dim as long nJoints
get #hFile,,nJoints
print "nJoints " & nJoints
if nJoints>0 then
  for i as integer = 0 to nJoints-1
    dim as single v(15)
    get #hFile,,v()
  next
end if

dim as long nVertexStreams
get #hFile,,nVertexStreams
print "nVertexStreams " & nVertexStreams

dim as long nVerticies
get #hFile,,nVerticies
print "nVerticies " & nVerticies
sleep

dim as CHUNK_ID cid
dim as long elementsize

for nV as integer = 0 to nVertexStreams-1
  get #hFile,,cid
  get #hFile,,elementsize
  select case as const cid
  case POSITON_ID         ' size 12 = 3 single * nVertices
    print "POSITON_ID"
    sleep
    dim as VertexPosition vposition
    for vP as integer = 0 to nVerticies-1
      get #hFile,,vposition
      print vp,vposition.x,vposition.y,vposition.z
    next
  case NORMAL_ID          ' size 6 = 3 short * nVertices
    print "NORMAL_ID"
    sleep
    dim as VertexNormal vvormal
    for vN as integer = 0 to nVerticies-1
      get #hFile,,vvormal
      print vN,vvormal.x/32767,vvormal.y/32767,vvormal.z/32767
    next
  case TANGENT_ID         ' size 6 = 3 short * nVertices
    print "TANGENT_ID"
    sleep
    dim as VertexTangent vtangent
    for vT as integer = 0 to nVerticies-1
      get #hFile,,vtangent
      print vT,vtangent.x/32767,vtangent.y/32767,vtangent.z/32767
    next
  case BITANGENTS_ID      ' size 6 = 3 short * nVertices
    print "BITANGENTS_ID"
    sleep
    dim as VertexBitangents vbitangents
    for vB as integer = 0 to nVerticies-1
      get #hFile,,vbitangents
      print vB,vbitangents.x/32767,vbitangents.y/32767,vbitangents.z/32767
    next
  case JOINT_INDEX_ID     ' size 4 up to 4 * ubyte (or 0 if unused)
    print "JOINT_INDEX_ID"
    sleep
    dim as VertexJointIndicies vJointIndicies
    for vJI as integer = 0 to nVerticies-1
      get #hFile,,vJointIndicies
      print vJI,vJointIndicies.a,vJointIndicies.b,vJointIndicies.c,vJointIndicies.c
    next
  case JOINT_WHEIGHTS_ID  ' size 4 = 4 * ubyte 
    print "JOINT_WHEIGHTS_ID"
    sleep
    dim as VertexJointWheights vJointWheights
    for vJW as integer = 0 to nVerticies-1
      get #hFile,,vJointWheights
      print vJW,vJointWheights.a/255,vJointWheights.b/255,vJointWheights.c/255,vJointWheights.c/255
    next
  case TEXTURE_COORDS0_ID ' size 8 = 2 single * nVertices texture coordinates set 0
    print "TEXTURE_COORDS0_ID"
    sleep
    dim as VertexTextureCoords vTexture0
    for vT0 as integer = 0 to nVerticies-1
      get #hFile,,vTexture0
      print vT0,vTexture0.u,vTexture0.v
    next
  case TEXTURE_COORDS1_ID ' size 8 = 2 single * nVertices texture coordinates set 1
    print "TEXTURE_COORDS1_ID"
    sleep
    dim as VertexTextureCoords vTexture1
    for vT1 as integer = 0 to nVerticies-1
      get #hFile,,vTexture1
      print vT1,vTexture1.u,vTexture1.v
    next
  case else
    close #hFile
    print "error: unknow VertexStream id " & cid
    beep : sleep : end
  end select
next

dim as long nTriangleIndicies
get #hFile,,nTriangleIndicies
dim as long nTriangles = nTriangleIndicies\3
print "nTriangleIndicies " & nTriangleIndicies & " = " & nTriangles & " Triangles" 
sleep
dim as TriangleIndex tIndexA,tIndexB,tIndexC
for nT as integer = 0 to nTriangles-1
  get #hFile,,tIndexA
  get #hFile,,tIndexB
  get #hFile,,tIndexC
  print nT,tIndexA,tIndexB,tIndexC
next

dim as long nMorphTargets
get #hFile,,nMorphTargets
print "nMorphTargets " & nMorphTargets

close #hFile
print
print "ok ..."
sleep










