#include once "fbgfx.bi"
#include once "horde3d.bi"

function InitHorde3D(byval w as integer=640,byval h as integer=480) as boolean
  dim as integer bits
  screeninfo ,,bits
  ' create window with OpenGL context
  screenres w,h,bits,,fb.GFX_OPENGL
  if screenptr()=0 then
    screen 0
    print "error: init OpenGL context !"
    beep : sleep : end 1
  end if
  flip
  ' init Horde3D
  var ret = h3dInit()
  if ret=false then h3dutDumpMessages()
  return ret
end function

if InitHorde3D(800,600)=false then
  screen 0
  print "error: init Horde3D !"
  beep:sleep:end
end if
' get current screen size
dim as integer w,h
screeninfo w,h

dim as string dumpFile  = command(0) & ".html"
dim as string dumpTitle = "Overlay test with: " & *h3dGetVersionString
h3dutSetDumpFile (strptr(dumpFile))
h3dutSetDumpTitle(strptr(dumpTitle))

' Add resources
var forwardPipeRes = h3dAddPipeline("pipelines/forward.pipeline.xml")
var fontMatRes     = h3dAddMaterial("overlays/font.material.xml")
var panelMatRes    = h3dAddMaterial("overlays/panel.material.xml")
var logoMatRes     = h3dAddMaterial("overlays/logo.material.xml")
var cubeScene      = h3dAddSceneGraph("models/cube/cube.scene.xml")
var skyBoxScene    = h3dAddSceneGraph("models/skybox/skybox.scene.xml")

' Load resources
var contentDir = exepath() + "/Content"
h3dutLoadResourcesFromDisk(strptr(contentDir))

' Add scene nodes
var skyboxNode = h3dAddNodes(H3DRootNode, skyboxScene)
h3dSetNodeTransform(skyboxNode, 0,0,0, 0,0,0, 200,200,200)
h3dSetNodeFlags    (skyboxNode, H3DNodeFlags_NoCastShadow, true )
var cubeNode = h3dAddNodes(H3DRootNode, cubeScene)

' Add camera
var cam = h3dAddCameraNode(H3DRootNode, "Camera", forwardPipeRes)

' Setup viewport and render target sizes
h3dSetNodeParamI( cam, H3DCamera_ViewportWidthI, w )
h3dSetNodeParamI( cam, H3DCamera_ViewportHeightI, h)
h3dResizePipelineBuffers( forwardPipeRes, w, h )
' Set camera parameters (pos, rotation, scale)
h3dSetNodeTransform(cam, 0,5,20, 0,0,0, 1,1,1)
' Set camera parameters (fov,  ratio, near, far)
h3dSetupCameraView( cam, 45.0, w / h, 0.1, 1000.0 )

' Add light source
var light = h3dAddLightNode(H3DRootNode, "Light1", 0, "LIGHTING", "SHADOWMAP")
h3dSetNodeTransform(light, 0,20,-5, 90*3,0,0, 1,1,1)
h3dSetNodeParamF(light, H3DLight_RadiusF, 0, 110)
h3dSetNodeParamF(light, H3DLight_FovF   , 0, 60)
h3dSetNodeParamI(light, H3DLight_ShadowMapCountI, 1)
h3dSetNodeParamF(light, H3DLight_ShadowMapBiasF, 0, 0.01)
h3dSetNodeParamF(light, H3DLight_ColorF3, 0, 0.8) ' red
h3dSetNodeParamF(light, H3DLight_ColorF3, 1, 0.8) ' green
h3dSetNodeParamF(light, H3DLight_ColorF3, 2, 0.8) ' blue


dim as single coords(...)=>{0.75, 0.8, 0, 1,  _
                            0.75, 1,   0, 0,  _
                            1   , 1,   1, 0,  _
                            1   , 0.8, 1, 1}

dim as single xRot,yRot,zRot
while inkey()=""
  ' transform the cube
  h3dSetNodeTransform(cubeNode,0,5,-10, xRot,yRot,zRot, 10,10,10)
  h3dutShowFrameStats(fontMatRes,panelMatRes,2)
  h3dShowOverlays(@coords(0), 4, 1,1,1,1, logoMatRes, 0)
  h3dutShowText( "text", 0, .9, 0.1, 1, 1, 1, fontMatRes)
  h3dRender( cam )
  h3dFinalizeFrame()
  h3dClearOverlays()
  ' swap buffers
  flip
  xRot += .3
  yRot += .6
  zRot += .9
wend

' free all resources
h3dRelease()
