#ifndef __tokamak_c_bi__
#define __tokamak_c_bi__

#ifndef NULL
#define NULL cptr(any ptr,0)
#endif

#ifdef __FB_WIN32__
 #libpath "lib/win" ' optional use import lib *.dll.a
#endif

#ifndef __FB_64BIT__
 #inclib "tokamak-c-32"  
#else
 #inclib "tokamak-c-64"  
#endif

type neV3
  declare constructor(byval x_ as single=0,byval y_ as single=0,byval z_ as single=0)
  declare operator  cast as single ptr
  as single x,y,z,pad
end type
constructor neV3(byval x_ as single,byval y_ as single,byval z_ as single)
  x=x_ : y=y_ : z=z_
end constructor

operator neV3.cast as single ptr
  operator = @this.x
end operator

type neQ
  as single x,y,z,w
end type

type neM3
  as neV3 M(2)
end type

type neT3
  as neM3 rot
  as neV3 pos
end type

const as long DEFAULT_RIGIDBODIES_COUNT          = 50   ' (50)
const as long DEFAULT_ANIMATEDBODIES_COUNT       = 50   ' (50)
const as long DEFAULT_RIGIDPARTICLES_COUNT       = 50   ' (50)
const as long DEFAULT_GEOMETRIES_COUNT           = 100  ' (50+50+50)
const as long DEFAULT_OVERLAPPED_PAIRS_COUNT     = (DEFAULT_GEOMETRIES_COUNT*(DEFAULT_GEOMETRIES_COUNT-1))\2
const as long DEFAULT_CONTROLLERS_COUNT          = 50   ' (50)
const as long DEFAULT_CONSTRAINTS_COUNT          = 100  ' (100)
const as long DEFAULT_CONTRAINT_SETS_COUNT       = 100  ' (100)
const as long DEFAULT_CONTRAINT_BUFFER_SIZE      = 2000 ' (2000)
const as long DEFAULT_SENSORS_COUNT              = 100  ' (100)
const as long DEFAULT_TERRAIN_NODES_START_COUNT  = 200  ' (200)
const as long DEFAULT_TERRAIN_NODES_GROWBY_COUNT = -1

type neSimulatorSizeInfo
  ' a neSimulatorSizeInfo are used to init the neSimulator
  declare constructor
  ' Number of rigid bodies in the simulation
  as long rigidBodiesCount
  ' Number of animated bodies in the simulation
  as long animatedBodiesCount
  ' Number of rigid particles in the simulation
  as long rigidParticleCount
  ' Number of controller instances in the simulation
  as long controllersCount
  ' Number of possible overlapping pairs.
  ' This has the maximum value of (n x (n - 1)) / 2, where n = rigidBodyCount + animatedBodyCount.
  ' But in practice it rarely reach that high. You can try to specify a smaller number to save memory.
  as long overlappedPairsCount
  ' Number of collision geometries in the simulator
  as long geometriesCount
  ' Number of joints in the simulation
  as long constraintsCount
  ' Number of joint Sets in the simulation
  as long constraintSetsCount
  ' Size of the buffer use to solve joints
  as long constraintBufferSize
  ' Number of sensors in the simulation
  as long sensorsCount
  ' Number of nodes use to store terrain triangles
  as long terrainNodesStartCount
  ' Grow by this size if run out of nodes
  ' -1 means: signify double the number of terrainNode, whenever the it reach full capacity.
  as long terrainNodesGrowByCount
end type

constructor neSimulatorSizeInfo
  ' The constructor function initialises the members to the default values.
  rigidBodiesCount        = DEFAULT_RIGIDBODIES_COUNT
  animatedBodiesCount     = DEFAULT_ANIMATEDBODIES_COUNT
  rigidParticleCount      = DEFAULT_RIGIDPARTICLES_COUNT
  controllersCount        = DEFAULT_CONTROLLERS_COUNT
  overlappedPairsCount    = DEFAULT_OVERLAPPED_PAIRS_COUNT
  geometriesCount         = DEFAULT_GEOMETRIES_COUNT
  constraintsCount        = DEFAULT_CONSTRAINTS_COUNT
  constraintSetsCount     = DEFAULT_CONTRAINT_SETS_COUNT
  constraintBufferSize    = DEFAULT_CONTRAINT_BUFFER_SIZE
  sensorsCount            = DEFAULT_SENSORS_COUNT
  terrainNodesStartCount  = DEFAULT_TERRAIN_NODES_START_COUNT
  terrainNodesGrowByCount = DEFAULT_TERRAIN_NODES_GROWBY_COUNT
end constructor

type neSimulator
  ' a neSimulator solves forces on bodies and handle collision between collision shapes
  as integer dummy
end type

type neCollisionTable
  ' Using the collision table, the application can specify the response whenever two bodies collide in the simulation.
  ' Each body in the simulation has a collision ID, which defaults to 0.
  ' A collision table contains a neResponseBitFlag for each pair of collision IDs.
  ' A neResponseBitFlag equal to RESPONSE_IGNORE means the collision will be ignored; no collision impulse or collision callback will be performed.
  ' The application can retrieve the neCollisionTable by calling neSimulatorGetCollisionTable function.
  as integer dummy
end type
type neReponseBitFlag as ulong
const as neReponseBitFlag RESPONSE_IGNORE           = 0
const as neReponseBitFlag RESPONSE_IMPULSE          = 1
const as neReponseBitFlag RESPONSE_CALLBACK         = 2
const as neReponseBitFlag RESPONSE_IMPULSE_CALLBACK = 3

type neGeometry
  ' a neGeometry descripts a collision shape and used by neAnimatedBody and neRigidBody
  ' (note: it exists one more collision shape a terrain)
  as integer dummy
end type

type neAnimatedBody
  ' it's normaly a static massless object with a collision neGeometry like a building ...
  ' and will never moved by the simulator (after a collision with a neRigidBody for example)
  ' "Animated" in this scope are a little bit misspelled
  ' however it means it can be animated (moved and or rotated) only by user
  ' a neAnimatedBody are fixed in space during a simulation step and can't collide with an other neAnimatedBody
  ' (note: you can define a geomtry as breakable if a defined limit of forces act on 
  ' it will be split in parts and converted to neRigidBody)
  as integer dummy
end type
type neBreakFlag as ulong
' The geometry does not break off upon collision
const as neBreakFlag NE_BREAK_DISABLE            = 0
' Normal breakage mode. Only this geometry will break when a suitably large collision has occurred.
const as neBreakFlag NE_BREAK_NORMAL             = 1
' When this geometry breaks off, all geometries within the current object, with break the flag not equal to NE_BREAK_DISABLE, will also break off.
const as neBreakFlag NE_BREAK_ALL                = 2
'  When this geometry breaks off, its neighbours within a certain radius will also break off. 
' This radius is specified by neGeometrySetBreakageNeighbourRadius.
const as neBreakFlag NE_BREAK_NEIGHBOUR          = 3
' the following are the same as the above, except that they specify the new object spawned from the geometry 
' will be a rigid particle, rather than a rigid body.
const as neBreakFlag NE_BREAK_NORMAL_PARTICLE    = 4
const as neBreakFlag NE_BREAK_ALL_PARTICLE       = 5
const as neBreakFlag NE_BREAK_NEIGHBOUR_PARTICLE = 6

type neRigidBody
  ' a neRgidBody has a mass and a collision neGeometry and will interact with all other rigid and animated bodies
  ' (one exption if a neRigidBody are created as a particle it will never collide with other particles)
  as integer dummy
end type

type neRigidBodyController
  ' a neRigidBodyController are used inside a optional user defined neRigidBodyCallback
  as integer dummy
end type

type neJoint
  ' a neJoint descipts the constrain of agility of an neRigidBody or the contrains between neRigidBodies
  ' (note the NE_JOINT_HINGE is the only joint that can be used as a motor axis)
  as integer dummy
end type

type neJointController
  ' a joint controller are used inside a optional user defined joint callback
  as integer dummy
end type

type neSensor
  ' neSensor is a line with position, length and a directition and can be added to neAnimated and neRigidBody.
  ' When the line representing the neSensor intersects with other bodies neGeometry shapes and the terrain (if any) 
  ' the intersection information can be retrieved and used by the application.
  as integer dummy 
end type

type neBodyType as ulong
const as neBodyType NE_TERRAIN       = 0
const as neBodyType NE_RIGID_BODY    = 1
const as neBodyType NE_ANIMATED_BODY = 2

' This structure is passed to the application during neCollisionCallback.
' It contains various blocks of information about a collision which has occurred during the current simulation step.
type neCollisionInfo
  union
  ' bodyA and bodyB are pointers to the bodies which are colliding.
  ' Their type depends on the value of typeA and typeB.
  ' If the value of typeB is NE_TERRAIN, then bodyB is NULL.
  as neRigidBody    ptr RigidBodyA
  as neAnimatedBody ptr AnimatedBodyA 
  end union
  union
  as neRigidBody    ptr RigidBodyB
  as neAnimatedBody ptr AnimatedBodyB 
  end union
  as neBodyType typeA
  as neBodyType typeB
  ' These are pointers to neGeometry objects involved in the collision, one from each of the body.
  ' If typeB equal to NE_TERRAIN, then geometryB is NULL
  as neGeometry ptr geometryA
  as neGeometry ptr geometryB
  ' The material indices of the geometries which are colliding.
  as long materialIdA
  as long materialIdB
  ' The contact points in body coordinates and in world coordinates.
  as neV3 bodyContactPointA    ' contact point A in body space of A
  as neV3 bodyContactPointB    ' contact point B in body space of B
  as neV3 worldContactPointA   ' contact point A in world space
  as neV3 worldContactPointB   ' contact point B in world space
  ' The relativeVelocity of the contact points. 
  as neV3 relativeVelocity
  ' The direction of the collision normal.
  as neV3 collisionNormal
end type

type neTraingleflag as ulong
const as neTraingleflag NE_TRI_TRIANGLE   = 0
const as neTraingleflag NE_TRI_HEIGHT_MAP = 1

type neTriangle
  ' a neTriangle is the shortes pice of a neTriangleMesh
  as long indices(2)
  as long materialID
  as neTraingleflag flag
  as any ptr userData
end type

type neTriangleMesh
  ' a neTriangleMesh is a collection of neTriangles and are used to describe a terrain
  ' the terrain are internaly stored in a quadtree the tree nodes are really fast 
  ' to detect collision with neRigidBodies and neSensors
  ' this makes it perfect to describe many static parts of your simualted world. 
  ' (ground, mountains, buildings, streets ...)
  declare function AddVertex(byval x as single,byval y as single,byval z as single) as integer
  declare function AddTriangle(byval a as integer,byval b as integer,byval c as integer) as integer
  as neV3 ptr       vertices
  as long           vertexCount
  as neTriangle ptr triangles
  as long           triangleCount
end type
function neTriangleMesh.AddVertex(byval x as single,byval y as single,byval z as single) as integer
  dim as integer index=-1
  if vertexCount>0 andalso vertices<>0 then
    for i as integer=0 to vertexCount-1
      if vertices[i].x=x andalso _
         vertices[i].y=y andalso _
         vertices[i].z=z then index=i : exit for
    next
    if index<>-1 then return index
  end if
  index = vertexCount : vertexCount+=1
  vertices=reallocate(vertices,vertexCount*sizeof(neV3))
  vertices[index].x=x
  vertices[index].y=y
  vertices[index].z=z
  return index
end function

function neTriangleMesh.AddTriangle(byval a as integer,byval b as integer,byval c as integer) as integer
  dim as integer index=-1
  if triangleCount>0 andalso triangles<>0 then
    for i as integer=0 to triangleCount-1
      if triangles[i].indices(0)=a andalso _
         triangles[i].indices(1)=b andalso _
         triangles[i].indices(2)=c then index=i : exit for
    next
    if index<>-1 then return index
  end if
  index = triangleCount : triangleCount+=1
  triangles=reallocate(triangles,triangleCount*sizeof(neTriangle))
  triangles[index].indices(0)=a
  triangles[index].indices(1)=b
  triangles[index].indices(2)=c
  triangles[index].materialID=0
  triangles[index].flag=NE_TRI_TRIANGLE
  triangles[index].userData=0  
  return index
end function  


type LOG_OUTPUT_LEVEL as ulong
const as LOG_OUTPUT_LEVEL LOG_OUTPUT_LEVEL_NONE = 0
const as LOG_OUTPUT_LEVEL LOG_OUTPUT_LEVEL_ONE  = 1
const as LOG_OUTPUT_LEVEL LOG_OUTPUT_LEVEL_FULL = 2

' ###########################################
' # optinal: user defined callbacks (CDECL) #
' ###########################################
type neCustomCDInfo
  ' the neCustomCDInfo struct must be filled inside of the optinal neCustomCDRB2RB and neCustomCDRB2AB callback's
  as neV3   collisionNormal
  as neV3   worldContactPointA
  as neV3   worldContactPointB
  as single penetrationDepth
  as long   materialIdA
  as long   materialIdB
end type
type neCustomCDRB2RBCallback        as function cdecl (byval bodyA              as neRigidBody ptr, _
                                                       byval bodyB              as neRigidBody ptr, _
                                                       byref cdInfo             as neCustomCDInfo) as boolean

type neCustomCDRB2ABCallback        as function cdecl (byval bodyA              as neRigidBody ptr, _
                                                       byval bodyB              as neAnimatedBody ptr, _
                                                       byref cdInfo             as neCustomCDInfo) as boolean

type neCollisionCallback            as sub      cdecl (byref collisionInfo      as neCollisionInfo)

type neTerrainTriangleQueryCallback as sub      cdecl (byref minBound           as const neV3, _
                                                       byref maxBound           as const neV3, _
                                                       byval candidateTriangles as long ptr ptr, _
                                                       byval triangles          as neTriangle ptr ptr, _
                                                       byval vertices           as neV3 ptr ptr, _
                                                       byval candidateCount     as long ptr, _
                                                       byval triangleCount      as long ptr, _
                                                       byval body               as neRigidBody ptr)

type neBreakageCallback             as sub      cdecl (byval originalBody       as any ptr, _
                                                       byval originalbodyType   as neBodyType, _
                                                       byval brokenGeometry     as neGeometry ptr, _
                                                       byval newBody            as neRigidBody)

type neRigidBodyControllerCallback  as sub      cdecl (byval controller         as neRigidBodyController ptr, _
                                                       byval timeStep           as single)

type neJointControllerCallback      as sub      cdecl (byval controller         as neJointController ptr, _
                                                       byval timeStep           as single)

type neLogOutputCallback            as sub      cdecl (byval logString as zstring ptr)


extern "c"


' ###############
' # neSimulator #
' ###############
' Use CreateSimulator to create a new instance of the simulator. 
' sizeInfo specifies the amount of various objects to allocate within the simulator.
declare function CreateSimulator(byref sizeInfo    as const neSimulatorSizeInfo=type<neSimulatorSizeInfo>, _
                                 byref gravitation as const neV3=type<neV3>(0,-9.81,0)) as neSimulator ptr 
' Use DestroySimulator to delete an existing instance of the simulator object.
declare sub      DestroySimulator(byref sim as neSimulator ptr)

declare function SimulatorGetGravity(byval sim as neSimulator ptr) as neV3
declare sub      SimulatorSetGravity(byval sim as neSimulator ptr, byref grav as const neV3)

' Use these functions to create or free a rigid body (neRigidBody), rigid particle (neRigidBody), or animated body (neAnimatedBody).
' When a body is freed, all their associated joints, geometries, sensors and callbacks are freed automatically.
declare function SimulatorCreateRigidBody(byval sim as neSimulator ptr) as neRigidBody ptr
declare function SimulatorCreateRigidParticle(byval sim as neSimulator ptr) as neRigidBody ptr
declare sub      SimulatorFreeRigidBody(byval sim as neSimulator ptr, byref body as neRigidBody ptr)

declare function SimulatorCreateAnimatedBody(byval sim as neSimulator ptr) as neAnimatedBody ptr
declare sub      SimulatorFreeAnimatedBody(byval sim as neSimulator ptr, byref body as neAnimatedBody ptr)

' Use this function to specify the terrain mesh to the simulator.
declare sub      SimulatorSetTerrainMesh(byval sim as neSimulator ptr, byval tris as neTriangleMesh ptr)
declare sub      SimulatorFreeTerrainMesh(byval sim as neSimulator ptr)

' Specify and retrieve the material properties for a corresponding material index.
' By default all materials have the following values: friction = 0.4 restitution = 0.5
declare function SimulatorSetMaterial(byval sim as neSimulator ptr, byval index as long, byval friction as single, byval restitution as single) as boolean
declare function SimulatorGetMaterial(byval sim as neSimulator ptr, byval index as long, byref friction as single, byref restitution as single) as boolean

' Advance the simulation by the amount sec, specified in milliseconds. nSteps specifies the number of sub-steps to perform.
declare sub      SimulatorAdvance(byval sim as neSimulator ptr, byval sec as single = 1/60)
declare sub      SimulatorAdvanceSteps(byval sim as neSimulator ptr, byval sec as single = 1/60, byval steps as long)
' don't use it if it's unclear for you
declare sub      SimulatorAdvanceMinMax(byval sim as neSimulator ptr, byval sec as single = 1/60, byval minTimeStep as single, byval maxTimeStep as single)

' Create and free a joint object.
' The first form of CreateJoint takes one neRigidBody.
' This means the joint constraint of the rigid body to the world.
' The second and third form require a joint which constrains bodyA to another rigid body or to an animated body respectively.
' When a rigid body or animated body is freed, all their associated joints, geometries, sensors and callbacks are freed automatically.
declare function SimulatorCreateRigidBodyWorldJoint(byval sim as neSimulator ptr, byval body as neRigidBody ptr) as neJoint ptr
declare function SimulatorCreateRigidBodyRigidBodyJoint(byval sim as neSimulator ptr, byval bodyA as neRigidBody ptr, byval bodyB as neRigidBody ptr) as neJoint ptr
declare function SimulatorCreateRigidBodyAnimatedBodyJoint(byval sim as neSimulator ptr, byval bodyA as neRigidBody ptr, byval bodyB as neAnimatedBody ptr) as neJoint ptr
declare sub      SimulatorFreeJoint(byval sim as neSimulator ptr, byref joint as neJoint ptr)

' simulator misc.
declare sub      SimulatorSetLogOutputLevel(byval sim as neSimulator ptr, byval lvl as LOG_OUTPUT_LEVEL)
declare function SimulatorGetCurrentSizeInfo(byval sim as neSimulator ptr) as neSimulatorSizeInfo
declare function SimulatorGetStartSizeInfo(byval sim as neSimulator ptr) as neSimulatorSizeInfo
declare sub      SimulatorGetMemoryAllocated(byval sim as neSimulator ptr, byref memoryAllocated as long)

declare function SimulatorGetIdleBodyCount(byval sim as neSimulator ptr) as long


' #######################
' # optional callback's #
' #######################
' Specify and retrieve the collision callback function.
' The collision callback function returns information about a collision between two objects in the simulation.
' Each pair of objects will only receive a maximum of one collision callback for each timestep of the simulation.
declare sub      SimulatorSetCollisionCallback(byval sim as neSimulator ptr, byval cb as neCollisionCallback)
declare function SimulatorGetCollisionCallback(byval sim as neSimulator ptr) as neCollisionCallback

' Specify and retrieve the breakage callback function.
' The breakage callback function is called by the simulator whenever a breakage is about to occur.
' The callback function is called for each geometry object which has broken.
' When the geometry is broken, a new rigid body (or rigid particle, depending the breakage flag) is spawned.
' During the breakage callback function the application can retrieve the pointer to the newly spawned object.
declare sub      SimulatorSetBreakageCallback(byval sim as neSimulator ptr, byval cb as neBreakageCallback)
declare function SimulatorGetBreakageCallback(byval sim as neSimulator ptr) as neBreakageCallback

declare sub      SimulatorSetTerrainTriangleQueryCallback(byval sim as neSimulator ptr, byval cb as neTerrainTriangleQueryCallback)
declare function SimulatorGetTerrainTriangleQueryCallback(byval sim as neSimulator ptr) as neTerrainTriangleQueryCallback

declare sub      SimulatorSetCustomCDRB2RBCallback(byval sim as neSimulator ptr, byval cb as neCustomCDRB2RBCallback)
declare function SimulatorGetCustomCDRB2RBCallback(byval sim as neSimulator ptr) as neCustomCDRB2RBCallback

declare sub      SimulatorSetCustomCDRB2ABCallback(byval sim as neSimulator ptr, byval cb as neCustomCDRB2ABCallback)
declare function SimulatorGetCustomCDRB2ABCallback(byval sim as neSimulator ptr) as neCustomCDRB2ABCallback

declare sub      SimulatorSetLogOutputCallback(byval sim as neSimulator ptr, byval cb as neLogOutputCallback)
declare function SimulatorGetLogOutputCallback(byval sim as neSimulator ptr) as neLogOutputCallback


' ####################
' # neCollisionTable #
' ####################
' Retrieve the pointer to the collision table.
' A collision table specifies the collision response between different collision groups inside the simulation.
declare function SimulatorGetCollisionTable(byval sim as neSimulator ptr) as neCollisionTable ptr
declare sub      CollisionTableSet(byval tbl as neCollisionTable ptr, byval collisionID1 as long, byval collisionID2 as long, byval response as neReponseBitFlag = RESPONSE_IMPULSE)
declare function CollisionTableGet(byval tbl as neCollisionTable ptr, byval collisionID1 as long, byval collisionID2 as long) as neReponseBitFlag
declare function CollisionTableGetMaxCollisionID(byval tbl as neCollisionTable ptr) as long

' ##############
' # neGeometry #
' ##############
' Specify and retrieve the user data associate with the neGeometry object.
declare sub      GeometrySetUserData(byval geo as neGeometry ptr, byval userData as any ptr)
declare function GeometryGetUserData(byval geo as neGeometry ptr) as any ptr

declare sub      GeometrySetCubeSize      (byval geo as neGeometry ptr, byval fSize as single)
declare sub      GeometrySetBoxSize       (byval geo as neGeometry ptr, byval fWidth as single, byval fHeight as single, byval fDepth as single)
declare sub      GeometrySetBoxSizeV3     (byval geo as neGeometry ptr, byref vSize as const neV3)
declare sub      GeometrySetSphereDiameter(byval geo as neGeometry ptr, byval fDiameter as single)
declare sub      GeometrySetCylinder      (byval geo as neGeometry ptr, byval fDiameter as single, byval fHeight as single)
#define GeometrySetCapsule GeometrySetCylinder

' return false if geo isn't a box, sphere or capsule
declare function GeometryGetBoxSize       (byval geo as neGeometry ptr, byref vSize as neV3) as boolean
declare function GeometryGetSphereDiameter(byval geo as neGeometry ptr, byref fDiameter as single) as boolean
declare function GeometryGetCylinder      (byval geo as neGeometry ptr, byref fDiameter as single, byref fHeight as single) as boolean
#define GeometryGetCapsule GeometryGetCylinder 

' Set and retrieve the transform of the geometry.
' The transform defines the geometry's position and orientation relative to its host body object.
' The transform is defined in the host body's space.
declare sub      GeometrySetTransform(byval geo as neGeometry ptr, byref t as neT3)
declare function GeometryGetTransform(byval geo as neGeometry ptr) as neT3

' Set and retrieve position and roation of the geometry. (rotation in degree)
declare sub      GeometrySetPositionRotation  (byval geo as neGeometry ptr, byval px as single, byval py as single, byval pz as single, byval rp as single, byval ry as single, byval rr as single)
declare sub      GeometryGetPositionRotation  (byval geo as neGeometry ptr, byref px as single, byref py as single, byref pz as single, byref rx as single, byref ry as single, byref rz as single)
declare sub      GeometrySetPositionRotationV3(byval geo as neGeometry ptr, byref position as const neV3, byref rotation as const neV3)
declare sub      GeometryGetPositionRotationV3(byval geo as neGeometry ptr, byref position as neV3, byref rotation as neV3)

' Set and retrieve the material index for the geometry.
' The material index refers to a material defined in the neSimulator using the function neSimulatorSetMaterial.
' During a collision, the materials of the two colliding geometries are used to calculate the correct collision response.
declare sub      GeometrySetMaterialIndex(byval geo as neGeometry ptr, byval index as long)
declare function GeometryGetMaterialIndex(byval geo as neGeometry ptr) as long

' Specify and retrieve the breaking behaviour of the geometry.
declare sub      GeometrySetBreakageFlag(byval geo as neGeometry ptr, byval flag as neBreakFlag)
declare function GeometryGetBreakageFlag(byval geo as neGeometry ptr) as neBreakFlag

' Specify and retrieve the mass and inertia mass tensor of the object spawned from the breakage.
declare sub      GeometrySetBreakageMass(byval geo as neGeometry ptr, byval mass as single)
declare function GeometryGetBreakageMass(byval geo as neGeometry ptr) as single
declare sub      GeometrySetBreakageInertiaTensor(byval geo as neGeometry ptr, byref tensor as const neV3)
declare function GeometryGetBreakageInertiaTensor(byval geo as neGeometry ptr) as neV3

' Specify and retrieve the magnitude of the collision impulse required to break this geometry.
declare sub      GeometrySetBreakageMagnitude(byval geo as neGeometry ptr, byval mag as single)
declare function GeometryGetBreakageMagnitude(byval geo as neGeometry ptr) as single

' Specify and retrieve the amount of absorption the breaking of this geometry has.
declare sub      GeometrySetBreakageAbsorption(byval geo as neGeometry ptr, byval absorption as single)
declare function GeometryGetBreakageAbsorption(byval geo as neGeometry ptr) as single

' Specify and retrieve the radius within which other geometries within the same body will also break when a breakage occurs.
declare sub      GeometrySetBreakageNeighbourRadius(byval geo as neGeometry ptr, byval radius as single)
declare function GeometryGetBreakageNeighbourRadius(byval geo as neGeometry ptr) as single

declare sub      GeometrySetBreakagePlane(byval geo as neGeometry ptr, byref planeNormal as const neV3)
declare function GeometryGetBreakagePlane(byval geo as neGeometry ptr) as neV3

declare sub      GeometryGetOpenGLMatrix (byval geo as neGeometry ptr, byval m4x4 as single ptr)
declare sub      GeometryGetOpenGLMatrixInverted(byval geo as neGeometry ptr, byval m4x4 as single ptr)
declare function GeometryRotateVector    (byval geo as neGeometry ptr, byref v as const neV3) as neV3
declare function GeometryTransformVector (byval geo as neGeometry ptr, byref v as const neV3) as neV3
declare sub      GeometryRotateVectors   (byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      GeometryTransformVectors(byval geo as neGeometry ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)

' ##################
' # neAnimatedBody #
' ##################
' Specify and retrieve the user data associate with the neAnimatedBody object.
declare sub      AnimatedBodySetUserData(byval ab as neAnimatedBody ptr, byval userData as any ptr)
declare function AnimatedBodyGetUserData(byval ab as neAnimatedBody ptr) as any ptr

' Set and retrieve spatial states for the animated body with these functions.
declare sub      AnimatedBodySetPosition  (byval ab as neAnimatedBody ptr, byval x as single, byval y as single, byval z as single)
declare sub      AnimatedBodyGetPosition  (byval ab as neAnimatedBody ptr, byref x as single, byref y as single, byref z as single)
declare sub      AnimatedBodySetPositionV3(byval ab as neAnimatedBody ptr,byref position as const neV3)
declare function AnimatedBodyGetPositionV3(byval ab as neAnimatedBody ptr) as neV3

' Set and retrieve position and roation of the animated body. (rotation in rad)
declare sub      AnimatedBodySetRotation  (byval ab as neAnimatedBody ptr, byval p as single, byval y as single, byval r as single)
declare sub      AnimatedBodyGetRotation  (byval ab as neAnimatedBody ptr, byref p as single, byref y as single, byref r as single)
declare sub      AnimatedBodySetRotationV3(byval ab as neAnimatedBody ptr, byref pyr as neV3)
declare function AnimatedBodyGetRotationV3(byval ab as neAnimatedBody ptr) as neV3

' Set and retrieve roation of the animated body b<uaternion and matrix.
declare sub      AnimatedBodySetRotationM3(byval ab as neAnimatedBody ptr, byref m as const neM3)
declare function AnimatedBodyGetRotationM3(byval ab as neAnimatedBody ptr) as neM3
declare sub      AnimatedBodySetRotationQ (byval ab as neAnimatedBody ptr, byref q as const neQ)
declare function AnimatedBodyGetRotationQ (byval ab as neAnimatedBody ptr) as neQ

' Set and retrieve position and roation of the animated body. (rotation in degree)
declare sub      AnimatedBodySetPositionRotation  (byval ab as neAnimatedBody ptr, byval px as single, byval py as single, byval pz as single, byval rp as single, byval ry as single, byval rr as single)
declare sub      AnimatedBodyGetPositionRotation  (byval ab as neAnimatedBody ptr, byref px as single, byref py as single, byref pz as single, byref rx as single, byref ry as single, byref rz as single)
declare sub      AnimatedBodySetPositionRotationV3(byval ab as neAnimatedBody ptr, byref position as const neV3, byref rotation as const neV3)
declare sub      AnimatedBodyGetPositionRotationV3(byval ab as neAnimatedBody ptr, byref position as neV3, byref rotation as neV3)

' Retrieve the transform of the geometry.
declare function AnimatedBodyGetTransform(byval ab as neAnimatedBody ptr) as neT3

' Set and retrieve the collision ID of the animated body. 
' The collision ID, along with the neCollisionTable, determines how the animated body 
' will respond when a collision has occured with other objects in the simulation.
declare sub      AnimatedBodySetCollisionID(byval ab as neAnimatedBody ptr, byval cid as long)
declare function AnimatedBodyGetCollisionID(byval ab as neAnimatedBody ptr) as long

' Use these functions to create and retrieve neGeometry objects for the animated body.
' The collection neGeometry, belonging to an animated body, defines the body's collision boundary.
' It also defines the animated body's breaking behaviour. 
' An animated body can also exist without any neGeomtry assigned to it; it just wont collide with anything in the simulation.
' All neGeometry objects within an animated body will be freed automatcally when the animated body is freed. See also neGeometry.
declare function AnimatedBodyAddGeometry     (byval ab as neAnimatedBody ptr) as neGeometry ptr
declare function AnimatedBodyGetGeometryCount(byval ab as neAnimatedBody ptr) as long
declare function AnimatedBodyRemoveGeometry  (byval ab as neAnimatedBody ptr, byval geo as neGeometry ptr) as boolean
declare sub      AnimatedBodyBeginIterateGeometry(byval ab as neAnimatedBody ptr)
declare function AnimatedBodyGetNextGeometry(byval ab as neAnimatedBody ptr) as neGeometry ptr

' Update the internal bounding information of the animated body. 
' Whenever the user changes the geometry and/or sensor of the rigid body 
' by adding, removing, or modifying the geometry and/or sensor, 
' this function needs to be called to update the internal bounding information.
declare sub      AnimatedBodyUpdateBoundingInfo(byval ab as neAnimatedBody ptr)

declare function AnimatedBodyBreakGeometry(byval ab as neAnimatedBody ptr, byval geo as neGeometry ptr) as neRigidBody  ptr

' Define whether or not an animated  body will collide with another body connected to it via joints (neJoint).
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable. 
declare sub      AnimatedBodySetCollideConnected(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetCollideConnected(byval ab as neAnimatedBody ptr) as boolean

declare sub      AnimatedBodySetCollideDirectlyConnected(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetCollideDirectlyConnected(byval ab as neAnimatedBody ptr) as boolean

declare sub      AnimatedBodySetUseCustomCollisionDetection(byval ab as neAnimatedBody ptr, byval yes as boolean,  byval obb as const neT3 ptr, byval boundingRadius as single)
declare function AnimatedBodyGetUseCustomCollisionDetection(byval ab as neAnimatedBody ptr) as boolean

' Use these functions to create and retrieve neSensor objects for the animated body. See neSensor.
declare function AnimatedBodyAddSensor(byval ab as neAnimatedBody ptr) as neSensor ptr
declare function AnimatedBodyRemoveSensor(byval ab as neAnimatedBody ptr, byval sen as neSensor ptr) as boolean
declare sub      AnimatedBodyBeginIterateSensor(byval ab as neAnimatedBody ptr)
declare function AnimatedBodyGetNextSensor(byval ab as neAnimatedBody ptr) as neSensor ptr

' Activate or deactivate the animated body. 
' When an animated body is deactivated, the simulator will not perform a collision test on the animated body.
' When activating (yes = true), an optional hint can be supplied.
' The hint should be a rigid body or an animated body near where the animated body being activated is.
' This can help speed up the introduction of the new activated animated body into the simulation.
declare sub      AnimatedBodySetActive(byval ab as neAnimatedBody ptr, byval yes as boolean)
declare function AnimatedBodyGetActive(byval ab as neAnimatedBody ptr) as boolean
declare sub      AnimatedBodySetActiveWithRigidBodyHint   (byval ab as neAnimatedBody ptr, byval yes as boolean, byval hint as neRigidBody    ptr)
declare sub      AnimatedBodySetActiveWithAnimatedBodyHint(byval ab as neAnimatedBody ptr, byval yes as boolean, byval hint as neAnimatedBody ptr)

declare sub      AnimatedBodyGetOpenGLMatrix(byval ab as neAnimatedBody ptr, byval m4x4 as single ptr)
declare sub      AnimatedBodyGetOpenGLMatrixInverted(byval ab as neAnimatedBody ptr, byval m4x4 as single ptr)
declare function AnimatedBodyRotateVector   (byval ab as neAnimatedBody ptr, byref v as const neV3) as neV3
declare function AnimatedBodyTransformVector(byval ab as neAnimatedBody ptr, byref v as const neV3) as neV3
declare sub      AnimatedBodyRotateVectors   (byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      AnimatedBodyTransformVectors(byval ab as neAnimatedBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)

' ###############
' # neRigidBody #
' ###############
' Specify and retrieve the user data associate with the neRigidBody object.
declare sub      RigidBodySetUserData(byval rb as neRigidBody ptr, byval userData as any ptr)
declare function RigidBodyGetUserData(byval rb as neRigidBody ptr) as any ptr 

' Sets and retrieves the mass of the rigid body.
' This property governs how fast a rigid body will accelerate under a given force, 
' as well as how big an impulse is needed to change the body's velocity.
declare sub      RigidBodySetMass(byval rb as neRigidBody ptr, byval mass as single)
declare function RigidBodyGetMass(byval rb as neRigidBody ptr) as single

' Set the body's mass inertia tensor. 
' This property is analagous to mass, but effects the rotational behaviour. 
' Use utility functions such as neBoxInertiaTensor to help you set this value.
declare sub      RigidBodySetInertiaTensor(byval rb as neRigidBody ptr, byref v as const neV3)
declare sub      RigidBodySetInertiaTensorM3(byval rb as neRigidBody ptr, byref tensor as const neM3)
declare sub      RigidBodyUpdateInertiaTensor(byval rb as neRigidBody ptr)

' Set and retrieve the collision ID of the rigid body.
' The collision ID, along with the neCollisionTable, determines how the rigid body will respond 
' when a collision has occurred with other objects in the simulation. See also neCollisionTable.
declare sub      RigidBodySetCollisionID(byval rb as neRigidBody ptr, byval cid as long)
declare function RigidBodyGetCollisionID(byval rb as neRigidBody ptr) as long

declare sub      RigidBodySetLinearDamping(byval rb as neRigidBody ptr, byval damp as single)
declare function RigidBodyGetLinearDamping(byval rb as neRigidBody ptr) as single

declare sub      RigidBodySetAngularDamping(byval rb as neRigidBody ptr, byval damp as single)
declare function RigidBodyGetAngularDamping(byval rb as neRigidBody ptr) as single

' Set and retrieve position of the rigid body.
declare sub      RigidBodySetPosition  (byval rb as neRigidBody ptr, byval x as single, byval y as single, byval z as single)
declare sub      RigidBodyGetPosition  (byval rb as neRigidBody ptr, byref x as single, byref y as single, byref z as single)
declare sub      RigidBodySetPositionV3(byval rb as neRigidBody ptr, byref p as const neV3)
declare function RigidBodyGetPositionV3(byval rb as neRigidBody ptr) as neV3

' Set and retrieve roation of the rigid body. (rotation in rad)
declare sub      RigidBodySetRotation(byval rb as neRigidBody ptr, byval p as single, byval y as single, byval r as single)
declare sub      RigidBodyGetRotation(byval rb as neRigidBody ptr, byref p as single, byref y as single, byref r as single)
declare sub      RigidBodySetRotationV3(byval rb as neRigidBody ptr, byref rotation as neV3)
declare function RigidBodyGetRotationV3(byval rb as neRigidBody ptr) as neV3

' Set and retrieve roation of the rigid body by quatrernion and matrix. 
declare sub      RigidBodySetRotationM3(byval rb as neRigidBody ptr, byref m as const neM3)
declare function RigidBodyGetRotationM3(byval rb as neRigidBody ptr) as neM3
declare sub      RigidBodySetRotationQ(byval rb as neRigidBody ptr, byref q as const neQ)
declare function RigidBodyGetRotationQ(byval rb as neRigidBody ptr) as neQ

' Set and retrieve position and roation of the rigid body. (rotation in degree)
declare sub      RigidBodySetPositionRotation  (byval rb as neRigidBody ptr, byval px as single, byval py as single, byval pz as single, byval rp as single, byval ry as single, byval rr as single)
declare sub      RigidBodyGetPositionRotation  (byval rb as neRigidBody ptr, byref px as single, byref py as single, byref pz as single,  byref rx as single, byref ry as single, byref rz as single)
declare sub      RigidBodySetPositionRotationV3(byval rb as neRigidBody ptr, byref position as const neV3, byref rotation as const neV3)
declare sub      RigidBodyGetPositionRotationV3(byval rb as neRigidBody ptr, byref position as neV3, byref rotation as neV3)

declare sub      RigidBodySetTransform(byval rb as neRigidBody ptr, byref t as neT3)
declare function RigidBodyGetTransform(byval rb as neRigidBody ptr) as neT3

' Set and retrieve dynamic states for the rigid body with these functions.
declare sub      RigidBodySetVelocity(byval rb as neRigidBody ptr, byref v as const neV3)
declare function RigidBodyGetVelocity(byval rb as neRigidBody ptr) as neV3

declare function RigidBodyGetVelocityAtPoint(byval rb as neRigidBody ptr, byref pt as const neV3) as neV3
declare function RigidBodyGetAngularVelocity(byval rb as neRigidBody ptr) as neV3

declare sub      RigidBodySetAngularMomentum(byval rb as neRigidBody ptr, byref am as const neV3)
declare function RigidBodyGetAngularMomentum(byval rb as neRigidBody ptr) as neV3

' Set and retrieve the force/torque acting on the rigid body.
' A force will accelerate the rigid body's velocity.
' A torque will accelerate the rigid body's angular velocity.
' When the force is applied at a point other than the Center of Mass of the rigid body, 
' it will also produce a torque, hence the second SetForce function, where position is defined in world coordinates.
' The force/torque will be apply continously. To apply no force/torque to the rigid body set the value to zero.
declare sub      RigidBodySetForceAtPos(byval rb as neRigidBody ptr, byref force as const neV3, byref position as const neV3)

declare sub      RigidBodySetForce(byval rb as neRigidBody ptr, byref force as const neV3)
declare function RigidBodyGetForce(byval rb as neRigidBody ptr) as neV3

declare sub      RigidBodySetTorque(byval rb as neRigidBody ptr, byref torque as const neV3)
declare function RigidBodyGetTorque(byval rb as neRigidBody ptr) as neV3

' Apply an impulse or a twist to the rigid body.
' An impulse will produce a immediate change in velocity.
' A twist will produce immediate a change in angular velocity.
' An impulse applied at a point other than the Center of Mass of the rigid body also produce a twist.
' To do this, use the second ApplyImpulse function, where position is define in world coordinates of the application point.
declare sub      RigidBodyApplyImpulse(byval rb as neRigidBody ptr, byref impulse as const neV3)
declare sub      RigidBodyApplyImpulseAtPos(byval rb as neRigidBody ptr, byref impulse as const neV3, byref position as const neV3)
declare sub      RigidBodyApplyTwist(byval rb as neRigidBody ptr, byref twist as const neV3)

' Enable or disable gravity for the rigid body.
' If gravity is enabled, the rigid body will have a force equal to gravity x mass
' automatically applied to it by the simulator.
declare sub      RigidBodyGravityEnable(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyIsGravityEnable(byval rb as neRigidBody ptr) as boolean

' Use these functions to create and retrieve neGeometry objects for the rigid body.
' The collection neGeometry belonging to a rigid body defines the body's collision boundary.
' It also defines the rigid body's breaking behaviour. 
' A rigid body can also exist and be simulated without any neGeomtry assigned to it; 
' it just won't collide with anything in the simulation. 
' All neGeometry objects within a rigid body will be freed automatcally when the rigid body is freed. See also neGeometry.
declare function RigidBodyGetGeometryCount(byval rb as neRigidBody ptr) as long
declare function RigidBodyAddGeometry(byval rb as neRigidBody ptr) as neGeometry ptr
declare function RigidBodyRemoveGeometry(byval rb as neRigidBody ptr, byval geo as neGeometry ptr) as boolean
declare sub      RigidBodyBeginIterateGeometry(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextGeometry(byval rb as neRigidBody ptr) as neGeometry ptr

' Update the internal bounding information of the rigid body.
' Whenever the user changes the geometry and/or sensor of the rigid body 
' by adding, removing, or modifying the geometry and/or sensor, 
' this function needs to be called to update the internal bounding information.
declare sub      RigidBodyUpdateBoundingInfo(byval rb as neRigidBody ptr)

' Breaks off the geometry object, which must belong to the rigid body. Returns the newly created rigid body.
declare function RigidBodyBreakGeometry(byval rb as neRigidBody ptr, byval geo as neGeometry ptr) as neRigidBody ptr

' Define whether or not a rigid body will collide with another body connected to it via joints (neJoint).
' The behaviour defined by this function overrides the behaviour defined by the neCollisionTable.
declare sub      RigidBodySetCollideConnected(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetCollideConnected(byval rb as neRigidBody ptr)  as boolean

declare sub      RigidBodySetCollideDirectlyConnected(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetCollideDirectlyConnected(byval rb as neRigidBody ptr) as boolean

declare sub      RigidBodySetUseCustomCollisionDetection(byval rb as neRigidBody ptr, byval yes as boolean, byval obb as const neT3 ptr, byval boundingRadius as single)
declare function RigidBodyGetUseCustomCollisionDetection(byval rb as neRigidBody ptr) as boolean

' sensors of the rigid body.
declare function RigidBodyAddSensor(byval rb as neRigidBody ptr) as neSensor ptr
declare function RigidBodyRemoveSensor(byval rb as neRigidBody ptr, byval sen as neSensor ptr) as boolean
declare sub      RigidBodyBeginIterateSensor(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextSensor(byval rb as neRigidBody ptr) as neSensor ptr

' activ state of the rigid body
declare sub      RigidBodySetSleepingParameter(byval rb as neRigidBody ptr, byval sleepingParam as single)
declare function RigidBodyGetSleepingParameter(byval rb as neRigidBody ptr) as single

' Activate or deactivate the rigid body.
' When a rigid body is deactivated, the simulator will not update its dynamic state, 
' will not call its controllers, and the rigid body will not collide with anything.
' When activating (yes = true), an optional hint can be supplied.
' The hint should be a rigid body or an animated body near where the rigid body being activated is.
' This can help speed up the introduction of the newly-activated rigid body into the simulation.
declare sub      RigidBodySetActive(byval rb as neRigidBody ptr, byval yes as boolean)
declare function RigidBodyGetActive(byval rb as neRigidBody ptr) as boolean
declare sub      RigidBodySetActiveWithRigidBodyHint   (byval rb as neRigidBody ptr, byval yes as boolean, byval hint as neRigidBody    ptr)
declare sub      RigidBodySetActiveWithAnimatedBodyHint(byval rb as neRigidBody ptr, byval yes as boolean, byval hint as neAnimatedBody ptr)

declare function RigidBodyIsIdle(byval rb as neRigidBody ptr) as boolean

declare sub      RigidBodyGetOpenGLMatrix(byval rb as neRigidBody ptr, byval m4x4 as single ptr)
declare sub      RigidBodyGetOpenGLMatrixInverted(byval rb as neRigidBody ptr, byval m4x4 as single ptr)
declare function RigidBodyRotateVector    (byval rb as neRigidBody ptr, byref v as const neV3) as neV3
declare function RigidBodyTransformVector (byval rb as neRigidBody ptr, byref v as const neV3) as neV3
declare sub      RigidBodyRotateVectors   (byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
declare sub      RigidBodyTransformVectors(byval rb as neRigidBody ptr, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)

' #########################
' # neRigidBodyController #
' #########################
declare function RigidBodyAddController(byval rb as neRigidBody ptr, byval rbccb as neRigidBodyControllerCallback, byval period as long=0) as neRigidBodyController ptr
declare function RigidBodyRemoveController(byval rb as neRigidBody ptr, byval controller as neRigidBodyController ptr) as boolean
declare sub      RigidBodyBeginIterateController(byval rb as neRigidBody ptr)
declare function RigidBodyGetNextController(byval rb as neRigidBody ptr) as neRigidBodyController ptr

' use this only inside a rigidBoy controller callback
declare function RigidBodyControllerGetRigidBody(byval controller as neRigidBodyController ptr) as neRigidBody ptr

declare sub      RigidBodyControllerSetForceWithTorque(byval controller as neRigidBodyController ptr, _
                                                       byval fx as single, byval fy as single, byval fz as single, _
                                                       byval px as single, byval py as single, byval pz as single)

declare sub      RigidBodyControllerSetForce(byval controller as neRigidBodyController ptr, byval fx as single, byval fy as single, byval fz as single)
declare sub      RigidBodyControllerGetForce(byval controller as neRigidBodyController ptr, byref fx as single, byref fy as single, byref fz as single)

declare sub      RigidBodyControllerSetTorque(byval controller as neRigidBodyController ptr, byval tx as single, byval ty as single, byval tz as single)
declare sub      RigidBodyControllerGetTorque(byval controller as neRigidBodyController ptr, byref tx as single, byref ty as single, byref tz as single)

declare sub      RigidBodyControllerSetForceWithTorqueV3(byval controller as neRigidBodyController ptr, _
                                                         byref force as const neV3, byref position as const neV3)

declare sub      RigidBodyControllerSetForceV3(byval controller as neRigidBodyController ptr, byref force as const neV3)
declare function RigidBodyControllerGetForceV3(byval controller as neRigidBodyController ptr) as neV3

declare sub      RigidBodyControllerSetTorqueV3(byval controller as neRigidBodyController ptr, byref  torque as const neV3)
declare function RigidBodyControllerGetTorqueV3(byval controller as neRigidBodyController ptr) as neV3

' ############
' # neSensor #
' ############
' Specify and retrieve the user data associate with the neSensor object.
declare sub      SensorSetUserData          (byval sensor as neSensor ptr, byval userData as any ptr)
declare function SensorGetUserData          (byval sensor as neSensor ptr) as any ptr
' Specify the position and end vector of the line representing the sensor object.
declare sub      SensorSetLineSensor        (byval sensor as neSensor ptr, byval px as single, byval py as single, byval pz as single, _
                                                                           byval ex as single, byval ey as single, byval ez as single)
declare sub      SensorSetLineSensorV3      (byval sensor as neSensor ptr, byref position as const neV3, byref lineVector as const neV3)
declare sub      SensorGetLineVector        (byval sensor as neSensor ptr, byref x as single, byref y as single, byref z as single)
declare function SensorGetLineVectorV3      (byval sensor as neSensor ptr) as neV3
' Retrieve the current position and direction of the sensor.
declare sub      SensorGetLinePosition      (byval sensor as neSensor ptr, byref x as single, byref y as single, byref z as single)
declare function SensorGetLinePositionV3    (byval sensor as neSensor ptr) as neV3

declare sub      SensorGetLineUnitVector    (byval sensor as neSensor ptr, byref x as single, byref y as single, byref z as single)
declare function SensorGetLineUnitVectorV3  (byval sensor as neSensor ptr) as neV3


' Use these functions to retrieve the current intersection information of the sensor.
' If GetDetectDepth is <= 0.0, then the sensor does not intersect with other geometries
' and all other values returned by GetDetectXXX functions are invalid.
declare function SensorGetDetectDepth         (byval sensor as neSensor ptr) as single

declare sub      SensorGetDetectNormal        (byval sensor as neSensor ptr, byref x as single, byref y as single, byref z as single)
declare function SensorGetDetectNormalV3      (byval sensor as neSensor ptr) as neV3
declare sub      SensorGetDetectContactPoint  (byval sensor as neSensor ptr, byref x as single, byref y as single, byref z as single)
declare function SensorGetDetectContactPointV3(byval sensor as neSensor ptr) as neV3
declare function SensorGetDetectRigidBody     (byval sensor as neSensor ptr) as neRigidBody ptr
declare function SensorGetDetectAnimatedBody  (byval sensor as neSensor ptr) as neAnimatedBody ptr
declare function SensorGetDetectMaterial      (byval sensor as neSensor ptr) as long


' ###########
' # neJoint #
' ###########
' Specify and retrieve the user data associate with the neJoint object.
declare sub      JointSetUserData(byval joint as neJoint ptr, byval userData as any ptr)
declare function JointGetUserData(byval joint as neJoint ptr) as any ptr

' Specify and retrieve the type of the joint
type neConstraintType as ulong
const as neConstraintType NE_JOINT_BALLSOCKET  = 0
const as neConstraintType NE_JOINT_HINGE       = 1
const as neConstraintType NE_JOINT_SLIDE       = 2
declare sub      JointSetType(byval joint as neJoint ptr, byval t as neConstraintType)
declare function JointGetType(byval joint as neJoint ptr) as neConstraintType

' Retrieve the bodies which are connected by this joint.
' The bodies are specified when the joint is created and cannot be changed.
declare function JointGetRigidBodyA(byval joint as neJoint ptr) as neRigidBody ptr
declare function JointGetRigidBodyB(byval joint as neJoint ptr) as neRigidBody ptr
declare function JointGetAnimatedBodyB(byval joint as neJoint ptr) as neAnimatedBody ptr

' Use these functions to specify and retrieve the reference frame of the joint.
' The reference frame of the joint specifies position and orientation 
' of the joint with respect to each body's local frame.
' For ball-and-socket joints, the position of the frame 
' specifies the location where the two bodies are connected.
' For hinge joints, the position specifies the location of the joint, 
' and the Y-axis of the joint frame is the axis of the joint 
' (which is the axis the two bodies are free to rotate about).
declare sub      JointSetFrameA(byval joint as neJoint ptr, byref frameA as const neT3)
declare function JointGetFrameA(byval joint as neJoint ptr) as neT3
declare sub      JointSetPositionRotationFrameA  (byval joint as neJoint ptr, byval px as single, byval py as single, byval pz as single, byval rp as single, byval ry as single, byval rr as single)
declare sub      JointSetPositionRotationV3FrameA(byval joint as neJoint ptr, byref position as const neV3, byref rotation as const neV3)

declare sub      JointSetFrameB    (byval joint as neJoint ptr, byref frameB as const neT3)
declare function JointGetFrameB    (byval joint as neJoint ptr) as neT3
declare sub      JointSetPositionRotationFrameB    (byval joint as neJoint ptr, byval px as single, byval py as single, byval pz as single, byval rp as single, byval ry as single, byval rr as single)
declare sub      JointSetPositionRotationV3FrameB    (byval joint as neJoint ptr, byref position as const neV3, byref rotation as const neV3)

' Use this function to specify the joint's frame in world coordinate. 
' To use this function, typically the two bodies of the joint would already be positioned correctly 
' in relation to each other in world co-ordinates.
declare sub      JointSetFrameWorld(byval joint as neJoint ptr, byref frame as const neT3)
declare sub      JointSetPositionRotationFrameWorld(byval joint as neJoint ptr, byval px as single, byval py as single, byval pz as single, byval rp as single, byval ry as single, byval rr as single)
declare sub      JointSetPositionRotationV3FrameWorld(byval joint as neJoint ptr, byref position as const neV3, byref rotation as const neV3)

' Specify and retrieve the length of the joint.
' Some joints, such as hinge joints, have length as well as position and orientation.
declare sub      JointSetJointLength(byval joint as neJoint ptr, byval length as single)
declare function JointGetJointLength(byval joint as neJoint ptr) as single

' Enable or disable the joint. The second function returns true if the joint is enabled.
declare sub      JointEnable(byval joint as neJoint ptr, byval yes as boolean)
declare function JointIsEnabled(byval joint as neJoint ptr) as boolean

declare sub      JointSetDampingFactor(byval joint as neJoint ptr, byval factor as single)
declare function JointGetDampingFactor(byval joint as neJoint ptr) as single

' These functions relate to the accuracy of the solution of the joint.
' Generally, lower accuracy results in faster simulation, 
' but may be prone to side effects if the accuracy is too low.
' Epsilon specifies the early-exit theshold for the constraint solver.
' Iteration relates to the number of iterations the solver will use to solve the constraint.
' Setting the epsilon to 0 means the solver will always perform 
' the specified number of iterations and will not exit early.
declare sub      JointSetEpsilon(byval joint as neJoint ptr, byval epsilon as single)
declare function JointGetEpsilon(byval joint as neJoint ptr) as single

declare sub      JointSetIteration(byval joint as neJoint ptr, byval iteration as long)
declare function JointGetIteration(byval joint as neJoint ptr) as long

' Set the upper and lower limits of the joint.
' Joint limits are used to restrict the movement of a joint.
' For ball-and-socket joints and hinge joints, the joint's current angle 
' is the angle between the X-axes of the joint's frame in each of the joint bodies.
' For ball-and-socket joints, the valid range of the lower limit is 0 to PI; 
' for the upper limit, between lower the limit and PI. 
' For a hinge joint, the valid range for lower and upper limits is -PI..PI, 
' but the upper limit must be greater than the lower limit.
' For ball and socket joints, a twist limit can also be specified. 
' This is done by calling the SetUpperLimit2 and SetLowerLimit2 functions.
declare sub      JointSetUpperLimit(byval joint as neJoint ptr, byval limit as single)
declare function JointGetUpperLimit(byval joint as neJoint ptr) as single
declare sub      JointSetLowerLimit(byval joint as neJoint ptr, byval limit as single)
declare function JointGetLowerLimit(byval joint as neJoint ptr) as single
' joint limits 2
declare sub      JointSetUpperLimit2(byval joint as neJoint ptr, byval limit as single)
declare function JointGetUpperLimit2(byval joint as neJoint ptr) as single
declare sub      JointSetLowerLimit2(byval joint as neJoint ptr, byval limit as single)
declare function JointGetLowerLimit2(byval joint as neJoint ptr) as single

' Enable or disable the joint limit. 
' The second function returns true if the joint limit is enabled. 
' The last two functions relate to the second limit for the joint. 
' Note: currently, only ball and socket joints have a second limit, which is the twist. 
declare sub      JointLimitEnable    (byval joint as neJoint ptr, byval yes as boolean)
declare function JointIsLimitEnabled (byval joint as neJoint ptr) as boolean
declare sub      JointLimit2Enable   (byval joint as neJoint ptr, byval yes as boolean) 
declare function JointIsLimit2Enabled(byval joint as neJoint ptr) as boolean

' joint motor
declare sub      JointMotorEnable(byval joint as neJoint ptr, byval yes as boolean)
declare function JointIsMotorEnabled(byval joint as neJoint ptr) as boolean
declare sub      JointSetMotor(byval joint as neJoint ptr, byval desireValue as single, byval maxForce as single)
declare sub      JointGetMotor(byval joint as neJoint ptr, byref desireValue as single, byref maxForce as single)

' #####################
' # neJointController #
' #####################
' These functions provide access to adding and removing joint controller callbacks.
declare function JointAddController(byval joint as neJoint ptr, byval jccb as neJointControllerCallback, byval period as long=0) as neJointController ptr
declare function JointRemoveController(byval joint as neJoint ptr, byval controller as neJointController ptr) as boolean

declare sub      JointBeginIterateController(byval joint as neJoint ptr)
declare function JointGetNextController(byval joint as neJoint ptr) as neJointController ptr

' use this only inside of a joint controller callback
declare function JointControllerGetJoint(byval controller as neJointController ptr) as neJoint ptr

declare sub      JointControllerSetForceV3BodyA(byval controller as neJointController ptr, byref  force as const neV3)
declare function JointControllerGetForceV3BodyA(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetForceV3BodyB(byval controller as neJointController ptr, byref  force as const neV3)
declare function JointControllerGetForceV3BodyB(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetTorqueV3BodyA(byval controller as neJointController ptr, byref  torque as const neV3)
declare function JointControllerGetTorqueV3BodyA(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetTorqueV3BodyB(byval controller as neJointController ptr, byref torque as const neV3)
declare function JointControllerGetTorqueV3BodyB(byval controller as neJointController ptr) as neV3

declare sub      JointControllerSetForceWithTorqueV3BodyA(byval controller as neJointController ptr, byref force as const neV3, byref pos as const neV3)
declare sub      JointControllerSetForceWithTorqueV3BodyB(byval controller as neJointController ptr, byref force as const neV3, byref pos as const neV3)

' ###########
' # helpers #
' ###########

' calculate the InertiaTensor for a Box
function neBoxInertiaTensor(byval fWidth  as single, _
                            byval fHeight as single, _
                            byval fDepth  as single, _
                            byval fMass   as single) as neV3
  dim as neV3 ret
  dim as single maxdim
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1
  if fMass  <1    then fMass  =1
  maxdim = fWidth
  if (fHeight > maxdim) then maxdim = fHeight
  if (fDepth  > maxdim) then maxdim = fDepth
  dim as single  xsq = maxdim
  dim as single  ysq = maxdim
  dim as single  zsq = maxdim

  fMass /= 3.0
  xsq *= xsq
  ysq *= ysq
  zsq *= zsq

  ret.x = (ysq + zsq) * fMass
  ret.y = (xsq + zsq) * fMass
  ret.z = (xsq + ysq) * fMass

  return ret
end function

' calculate the InertiaTensor for a Box
function neBoxInertiaTensorV3(byref vSize as const neV3, byval fMass as single) as neV3
  return neBoxInertiaTensor(vSize.x, vSize.y, vSize.z, fMass)
end function

' calculate the InertiaTensor for a Cube
function neCubeInertiaTensor(byval fSize as single, byval fMass as single) as neV3
  return neBoxInertiaTensor(fSize, fSize, fSize, fMass)
end function

' calculate the InertiaTensor of a Sphere
function neSphereInertiaTensor(byval fDiameter as single, byval fMass as single) as neV3
  dim as neV3 ret
  dim as single radius
  dim as single value
  if fDiameter<0.1 then fDiameter=0.1
  if fMass<1 then fMass=1
  radius = fDiameter * 0.5
  value = 2.0 / 5.0 * fMass * radius * radius
  ret.x=value
  ret.y=value
  ret.z=value
  return ret
end function

' calculate the InertiaTensor of a Cylinder
#define neCapsuleInertiaTensor neCylinderInertiaTensor
function neCylinderInertiaTensor(byval fDiameter as single, _
                                 byval fHeight   as single, _
                                 byval fMass     as single) as neV3
  dim as neV3 ret
  dim as single radius
  dim as single radiusSq
  dim as single Ixz
  dim as single Iyy

  if fDiameter<0.1 then fDiameter=0.1
  if fHeight  <0.1 then fHeight=0.1
  if fMass    <1    then fMass=1

  radius = fDiameter * 0.5
  radiusSq = radius * radius
  Ixz = 1.0 / 12.0 * fMass * fHeight * fHeight + 0.25 * fMass * radiusSq
  Iyy = 0.5 * fMass * radiusSq

  ret.x = Ixz
  ret.y = Iyy
  ret.z = Ixz
  return ret
end function

' create a static massless body with collision geometry
' !!! can only animated by the user !!!
function CreateAnimatedBodyBox(byval sim     as neSimulator ptr, _
                               byval fWidth  as single=1, _
                               byval fHeight as single=1, _
                               byval fDepth  as single=1) as neAnimatedBody ptr
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1 
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function

function CreateAnimatedBodyCube(byval sim as neSimulator ptr, _
                                byval fSize as single = 1) as neAnimatedBody ptr
  return CreateAnimatedBodyBox(sim,fSize,fSize,fSize)
end function  

function CreateAnimatedBodySphere(byval sim       as neSimulator ptr, _
                                  byval fDiameter as single=1) as neAnimatedBody ptr
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fDiameter <0.1 then fDiameter=0.1
  GeometrySetSphereDiameter(geo, fDiameter)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function
#define CreateAnimatedBodyCapsule CreateAnimatedBodyCylinder
function CreateAnimatedBodyCylinder(byval sim       as neSimulator ptr, _
                                    byval fDiameter as single=0.5, _
                                    byval fHeight   as single=1) as neAnimatedBody ptr
  var ab  = SimulatorCreateAnimatedBody(sim)
  var geo = AnimatedBodyAddGeometry(ab)
  if fDiameter<0.1 then fDiameter=0.1
  if fHeight  <0.1 then fHeight = 0.1
  GeometrySetCylinder(geo, fDiameter, fHeight)
  AnimatedBodyUpdateBoundingInfo(ab)
  return ab
end function


' create a dynamic body with mass and collision geometry
function CreateRigidBodyBox(byval sim     as neSimulator ptr, _
                            byval fWidth  as single=1, _
                            byval fHeight as single=1, _
                            byval fDepth  as single=1, _
                            byval fMass   as single=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1
  if fMass  <1   then fMass  =1
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  RigidBodySetInertiaTensor(rb,neBoxInertiaTensor(fWidth,fHeight,fDepth,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

function CreateRigidBodyCube(byval sim   as neSimulator ptr, _
                             byval fSize as single=1, _
                             byval fMass as single=1) as neRigidBody ptr
  return CreateRigidBodyBox(sim,fSize,fsize,fSize,fMass)
end function  


function CreateRigidBodySphere(byval sim       as neSimulator ptr, _
                               byval fDiameter as single=1, _
                               byval fMass     as single=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter<0.1 then fDiameter =0.1
  if fMass  <1    then fMass  =1
  GeometrySetSphereDiameter(geo, fDiameter)
  RigidBodySetInertiaTensor(rb,neSphereInertiaTensor(fDiameter,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

#define CreateRigidBodyCapsule CreateRigidBodyCylinder
function CreateRigidBodyCylinder(byval sim       as neSimulator ptr, _
                                 byval fDiameter as single=0.5, _
                                 byval fHeight   as single=1, _
                                 byval fMass     as single=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidBody(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter < 0.1 then fDiameter = 0.1
  if fHeight   < 0.1 then fHeight   = 0.1
  if fMass     < 1    then fMass     = 1
  GeometrySetCylinder(geo, fDiameter, fHeight)
  RigidBodySetInertiaTensor(rb,neCylinderInertiaTensor(fDiameter,fHeight,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function


' Create a dynamic particle with mass and collision geometry
function CreateRigidParticleBox(byval sim     as neSimulator ptr, _
                                byval fWidth  as single=1, _
                                byval fHeight as single=1, _
                                byval fDepth  as single=1, _
                                byval fMass   as single=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fWidth <0.1 then fWidth =0.1
  if fHeight<0.1 then fHeight=0.1
  if fDepth <0.1 then fDepth =0.1
  if fMass  <1    then fMass  =1
  GeometrySetBoxSize(geo, fWidth,fHeight,fDepth)
  RigidBodySetInertiaTensor(rb,neBoxInertiaTensor(fWidth,fHeight,fDepth,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

function CreateRigidParticleCube(byval sim   as neSimulator ptr, _
                                 byval fSize as single=1, _
                                 byval fMass as single=1) as neRigidBody ptr
  return CreateRigidParticleBox(sim,fSize,fSize,fSize,fMass)
end function


function CreateRigidParticleSphere(byval sim       as neSimulator ptr, _
                                   byval fDiameter as single=1, _
                                   byval fMass     as single=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter<0.1 then fDiameter =0.1
  if fMass    <1   then fMass  =1
  GeometrySetSphereDiameter(geo, fDiameter)
  RigidBodySetInertiaTensor(rb,neSphereInertiaTensor(fDiameter,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function

#define CreateRigidParticleCapsule CreateRigidParticleCylinder
function CreateRigidParticleCylinder(byval sim       as neSimulator ptr, _
                                     byval fDiameter as single=0.5, _
                                     byval fHeight   as single=1, _
                                     byval fMass     as single=1) as neRigidBody ptr
  var rb  = SimulatorCreateRigidParticle(sim)
  var geo = RigidBodyAddGeometry(rb)
  if fDiameter < 0.1 then fDiameter = 0.1
  if fHeight   < 0.1 then fHeight   = 0.1
  if fMass     < 1   then fMass     = 1
  GeometrySetCylinder(geo, fDiameter, fHeight)
  RigidBodySetInertiaTensor(rb,neCylinderInertiaTensor(fDiameter,fHeight,fMass))
  RigidBodySetMass(rb,fMass)
  RigidBodyUpdateBoundingInfo(rb)
  return rb
end function


' get pitch yaw roll from matrix
declare function RotAngle(byref m as neM3) as neV3
' create matrix from pitch yaw roll vector
declare function RotMatrixV3(byref v as neV3) as neM3
' create matrix from pitch yaw roll 
declare function RotMatrix(byval p as single, byval y as single, byval r as single) as neM3
' get OpenGL matrix from frame
declare sub      FrameGetOpenGLMatrix(byref frame as neT3, byval m4x4 as single ptr)
' get the inverse of the OpenGL matrix from frame
declare sub      FrameGetOpenGLMatrixInverted(byref frame as neT3, byval m4x4 as single ptr)
' use the frame to rotate a vector
declare function FrameRotateVector    (byref frame as neT3, byref v as const neV3) as neV3
' use the frame to transform a vector
declare function FrameTransformVector (byref frame as neT3, byref v as const neV3) as neV3
' use the frame to rotate an array of vectors
declare sub      FrameRotateVectors   (byref frame as neT3, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)
' use the frame to transform an array of vectors
declare sub      FrameTransformVectors(byref frame as neT3, byval dst as neV3 ptr, byval src as const neV3 ptr, byval nVectors as long)

declare sub      V3SetZero(byref v as neV3)
declare function V3Add(byref a as const neV3, byref b as const neV3) as neV3
declare function V3Sub(byref a as const neV3, byref b as const neV3) as neV3
declare function V3Mul(byref v as const neV3, byval s as single) as neV3
declare function V3Dot(byref a as const neV3, byref b as const neV3) as single
declare function V3Length(byref v as const neV3) as single
declare function V3Distance(byref a as const neV3,byref b as const neV3) as single
declare sub      V3Normalize(byref v as neV3)
declare function V3Normalized(byref v as const neV3) as neV3
declare function V3Cross(byref a as const neV3, byref b as const neV3) as neV3
declare sub      V3RemoveComponent(byref v as neV3, byref other as const neV3)
declare function V3Lerp(byref a as const neV3, byref b as const neV3, byval t as single) as neV3
declare function V3Mix (byref a as const neV3, byref b as const neV3, byval t as single) as neV3

declare sub      M3SetIdentiy(byref m as neM3)
declare function M3MulV3(byref m as const neM3, byref v as const neV3) as neV3
declare function M3MulM3(byref a as const neM3, byref b as const neM3) as neM3
declare function M3FromQ(byref q as const neQ) as neM3
declare function M3FromAxisAngle(byref axis as const neV3, byref angle as const single) as neM3
declare function M3FromEulerAngle(byval p as single, byval y as single, byval r as single) as neM3
declare function M3Lerp(byref a as const neM3,byref b as const neM3,byval t as single) as neM3

declare sub      T3SetIdentiy(byref t as neT3)
declare function T3MulV3(byref t as const neT3, byref v as const neV3) as neV3
declare function T3MulT3(byref a as const neT3, byref b as const neT3) as neT3
declare function T3Inverted(byref t as const neT3) as neT3
declare sub      T3Inverse(byref t as neT3)

declare sub      QNormalize(byref q as neQ)
declare function QNormalized(byref q as neQ) as neQ
declare function QFromM3(byref m as const neM3) as neQ
declare function QMulQ(byref a as const neQ, byref b as const neQ) as neQ
declare function QFromAxisAngle(byref axis as const neV3, byval angle as single) as neQ
declare sub      QGetAxisAngle(byref q as const neQ, byref axis as neV3, byref angle as single)
declare function QLerp(byref a as neQ, byref b as neQ,byval t as single) as neQ

end extern



#endif ' __tokamak_c_bi__ 